#include "t3f/t3f.h"
#include "t3f/draw.h"
#include "t3f/menu.h"

#include "core/avatar.h"
#include "core/resource.h"

#include "main.h"
#include "init.h"
#include "ui_proc.h"
#include "undo.h"
#include "flood_fill.h"

static char * preview_text[4] = {"Normal", "Happy", "Pain", "Upset"};

static bool app_hover_selection(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	float mx, my;

	if(app->selection_active)
	{
		mx = (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM;
		my = (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM;
		if(mx >= app->float_left && mx < app->float_right && my >= app->float_top && my < app->float_bottom)
		{
			return true;
		}
	}
	return false;
}

void app_unselect(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_STATE old_state;
	ALLEGRO_TRANSFORM identity;

	if(app->float_bitmap && app->float_original)
	{
		al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
		al_set_target_bitmap(app->float_bp);
		al_identity_transform(&identity);
		al_use_transform(&identity);
		al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
		al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
		al_draw_bitmap(app->float_original, 0, 0, 0);
		al_draw_filled_rectangle(app->selection_left, app->selection_top, app->selection_right, app->selection_bottom, al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
		al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
		al_draw_bitmap(app->float_bitmap, app->float_left, app->float_top, 0);
		al_restore_state(&old_state);
	}
	if(app->float_bitmap)
	{
		al_destroy_bitmap(app->float_bitmap);
		app->float_bitmap = NULL;
	}
	if(app->float_original)
	{
		al_destroy_bitmap(app->float_original);
		app->float_original = NULL;
	}
	app->selection_active = false;
}

void app_update_window_title(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	char title[1024] = {0};

	if(app->avatar)
	{
		snprintf(title, 1024, "%s%s - vGolf Deluxe Avatar Editor", app->changes ? "*" : "", app->last_open_path ? al_get_path_filename(app->last_open_path) : "Untitled");
		al_set_window_title(t3f_display, title);
	}
}

void app_event_handler(ALLEGRO_EVENT * event, void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	if(event->type == ALLEGRO_EVENT_DISPLAY_CLOSE)
	{
		if(!t3gui_get_active_dialogs())
		{
			ui_proc_try_exit(0, data);
		}
	}
	else if(event->type == ALLEGRO_EVENT_DISPLAY_RESIZE)
	{
		t3f_event_handler(event);
		t3gui_center_dialog(app->unsaved_dialog, al_get_display_width(t3f_display), al_get_display_height(t3f_display));
		t3gui_center_dialog(app->tags_dialog, al_get_display_width(t3f_display), al_get_display_height(t3f_display));
		t3gui_center_dialog(app->about_dialog, al_get_display_width(t3f_display), al_get_display_height(t3f_display));
	}
	else
	{
		t3f_event_handler(event);
	}
}

static bool app_shortcut(int shortcut_key, int modifier_key, int exclude_key, int (*proc)(int id, void * data), void * data)
{
	if(t3f_key[shortcut_key] && (!modifier_key || t3f_key[modifier_key]) && (!exclude_key || !t3f_key[exclude_key]))
	{
		proc(0, data);
		t3f_key[shortcut_key] = 0;
		return true;
	}
	return false;
}

static void app_shortcut_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	int i;

	app_shortcut(ALLEGRO_KEY_N, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_file_new, data);
	app_shortcut(ALLEGRO_KEY_O, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_file_open, data);
	app_shortcut(ALLEGRO_KEY_Q, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_file_exit, data);
	if(app->avatar)
	{
		app_shortcut(ALLEGRO_KEY_S, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_file_save, data);
		app_shortcut(ALLEGRO_KEY_A, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_file_save_as, data);

		/* edit menu */
		if(undo_available(data))
		{
			app_shortcut(ALLEGRO_KEY_Z, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_edit_undo, data);
		}
		if(redo_available(data))
		{
			app_shortcut(ALLEGRO_KEY_B, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_edit_redo, data);
		}
		app_shortcut(ALLEGRO_KEY_X, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_edit_cut, data);
		app_shortcut(ALLEGRO_KEY_C, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_edit_copy, data);
		app_shortcut(ALLEGRO_KEY_V, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_edit_paste, data);

		/* image menu */
		app_shortcut(ALLEGRO_KEY_A, 0, ALLEGRO_KEY_LCTRL, ui_menu_proc_image_shift_left, data);
		app_shortcut(ALLEGRO_KEY_D, 0, ALLEGRO_KEY_LCTRL, ui_menu_proc_image_shift_right, data);
		app_shortcut(ALLEGRO_KEY_W, 0, ALLEGRO_KEY_LCTRL, ui_menu_proc_image_shift_up, data);
		app_shortcut(ALLEGRO_KEY_S, 0, ALLEGRO_KEY_LCTRL, ui_menu_proc_image_shift_down, data);
		app_shortcut(ALLEGRO_KEY_H, 0, ALLEGRO_KEY_LCTRL, ui_menu_proc_image_mirror, data);
		app_shortcut(ALLEGRO_KEY_V, 0, ALLEGRO_KEY_LCTRL, ui_menu_proc_image_flip, data);
		app_shortcut(ALLEGRO_KEY_RIGHT, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_image_turn_cw, data);
		app_shortcut(ALLEGRO_KEY_LEFT, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_image_turn_ccw, data);
		app_shortcut(ALLEGRO_KEY_L, ALLEGRO_KEY_LCTRL, 0, ui_menu_proc_image_load, data);

		/* image selection */
		if(!t3f_key[ALLEGRO_KEY_LCTRL])
		{
			if(t3f_key[ALLEGRO_KEY_LEFT])
			{
				app->selected_image--;
				if(app->selected_image < 0)
				{
					app->selected_image = 3;
				}
				app_unselect(data);
				t3f_key[ALLEGRO_KEY_LEFT] = 0;
			}
			if(t3f_key[ALLEGRO_KEY_RIGHT])
			{
				app->selected_image++;
				if(app->selected_image > 3)
				{
					app->selected_image = 0;
				}
				app_unselect(data);
				t3f_key[ALLEGRO_KEY_RIGHT] = 0;
			}
		}

		for(i = 0; i < 9; i++)
		{
			if(t3f_key[ALLEGRO_KEY_1 + i])
			{
				app->selected_tool = i;
				app_unselect(data);
				t3f_key[ALLEGRO_KEY_1 + i] = 0;
			}
		}
	}

	/* help menu */
	app_shortcut(ALLEGRO_KEY_F1, 0, ALLEGRO_KEY_LCTRL, ui_menu_proc_help_manual, data);
}

void app_ui_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	unsigned char r, g, b;
	int i;

	if(t3gui_get_active_dialogs())
	{
		t3gui_logic();
		return;
	}
	app->hover_color = al_map_rgba_f(0.0, 0.0, 0.0, 0.0);

	app_shortcut_logic(data);

	/* interact with tools */
	if(t3f_mouse_y < UI_OFFSET_Y_EDITOR)
	{
		if(t3f_mouse_button[0])
		{
			app->selected_tool = (int)t3f_mouse_x / 32;
			if(app->selected_tool > 8)
			{
				app->selected_tool = 8;
			}
			app_unselect(data);
		}
	}

	/* interact with palette */
	else if(t3f_mouse_x >= UI_OFFSET_X_PALETTE && t3f_mouse_x < UI_OFFSET_X_PALETTE + 8 * 8 && t3f_mouse_y >= UI_OFFSET_Y_PALETTE && t3f_mouse_y < UI_OFFSET_Y_PALETTE + 8 * 32)
	{
		app->hover_color = al_get_pixel(app->palette_bitmap, t3f_mouse_x - UI_OFFSET_X_PALETTE, t3f_mouse_y - UI_OFFSET_Y_PALETTE);
		al_unmap_rgb(app->hover_color, &r, &g, &b);
		if(r == 255 && g == 0 && b == 255)
		{
			app->hover_color = al_map_rgba_f(0.0, 0.0, 0.0, 0.0);
		}
		if(t3f_mouse_button[0])
		{
			app->selected_color[0] = app->hover_color;
		}
		else if(t3f_mouse_button[1])
		{
			app->selected_color[1] = app->hover_color;
		}
	}

	if(app->avatar)
	{
		if(t3f_mouse_x >= UI_OFFSET_X_EDITOR && t3f_mouse_x < UI_OFFSET_X_EDITOR + 64 * 5 && t3f_mouse_y >= UI_OFFSET_Y_EDITOR && t3f_mouse_y < UI_OFFSET_Y_EDITOR + 58 * 5)
		{
			if(app_hover_selection(data))
			{
				al_set_system_mouse_cursor(t3f_display, ALLEGRO_SYSTEM_MOUSE_CURSOR_MOVE);
				if(t3f_mouse_button[0])
				{
					app->peg_x = (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM;
					app->peg_y = (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM;

					/* float selection if we haven't already */
					if(!app->float_bitmap)
					{
						ALLEGRO_STATE old_state;
						ALLEGRO_TRANSFORM identity;

						app->float_bp = app->avatar->image[app->selected_image]->bitmaps->bitmap[0];
						app->float_original = al_clone_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0]);
						app->float_bitmap = al_create_bitmap(app->selection_right - app->selection_left, app->selection_bottom - app->selection_top);
						if(app->float_bitmap)
						{
							undo_store(app);
							t3f_refresh_menus();
							al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
							al_set_target_bitmap(app->float_bitmap);
							al_identity_transform(&identity);
							al_use_transform(&identity);
							al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
							al_draw_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0], -app->selection_left, -app->selection_top, 0);
							al_restore_state(&old_state);
						}
					}
					app->state = STATE_FLOAT;
				}
			}
			else
			{
				al_set_system_mouse_cursor(t3f_display, ALLEGRO_SYSTEM_MOUSE_CURSOR_DEFAULT);
				for(i = 0; i < 2; i++)
				{
					if(t3f_mouse_button[i])
					{
						if(app->selected_tool != TOOL_SELECT)
						{
							undo_store(app);
							t3f_refresh_menus();
						}
						app->draw_color = app->selected_color[i];
						app->peg_x = (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM;
						app->peg_y = (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM;
						app_unselect(data);
						app->tool_blocked = false;
						app->state = STATE_DRAW;
					}
				}
			}
		}
		else if(t3f_mouse_y >= UI_OFFSET_Y_PREVIEW && t3f_mouse_x < UI_SPACE_PREVIEW * 4)
		{
			if(t3f_mouse_button[0])
			{
				app->selected_image = (int)t3f_mouse_x / UI_SPACE_PREVIEW;
				app_unselect(data);
			}
		}
	}
}

void app_draw_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_STATE old_state;
	ALLEGRO_TRANSFORM identity;
	int val;

	switch(app->selected_tool)
	{
		case TOOL_PAINT:
		{
			if(t3f_mouse_x >= UI_OFFSET_X_EDITOR && t3f_mouse_x < UI_OFFSET_X_EDITOR + 64 * UI_ZOOM && t3f_mouse_y >= UI_OFFSET_Y_EDITOR && t3f_mouse_y < UI_OFFSET_Y_EDITOR + 58 * UI_ZOOM)
			{
				al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
				al_set_target_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0]);
				al_put_pixel((int)(t3f_mouse_x - UI_OFFSET_X_EDITOR) / 5, (int)(t3f_mouse_y - UI_OFFSET_Y_EDITOR) / 5, app->draw_color);
				al_set_target_bitmap(app->scratch_bitmap);
				al_identity_transform(&identity);
				al_use_transform(&identity);
				al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
				al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
				al_draw_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0], 0, 0, 0);
				al_restore_state(&old_state);
			}
			break;
		}
		case TOOL_LINE:
		{
			al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
			al_set_target_bitmap(app->scratch_bitmap);
			al_identity_transform(&identity);
			al_use_transform(&identity);
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
			al_draw_line(app->peg_x + 0.5, app->peg_y + 0.5, (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM + 1.0 + 0.5, (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM + 1.0 + 0.5, app->draw_color, 1.0);
			al_restore_state(&old_state);
			break;
		}
		case TOOL_RECTANGLE:
		{
			al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
			al_set_target_bitmap(app->scratch_bitmap);
			al_identity_transform(&identity);
			al_use_transform(&identity);
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
			al_draw_rectangle(app->peg_x + 0.5, app->peg_y + 0.5, (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM + 0.5, (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM + 0.5, app->draw_color, 1.0);
			al_restore_state(&old_state);
			break;
		}
		case TOOL_FILLED_RECTANGLE:
		{
			al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
			al_set_target_bitmap(app->scratch_bitmap);
			al_identity_transform(&identity);
			al_use_transform(&identity);
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
			al_draw_filled_rectangle(app->peg_x + 0.5, app->peg_y + 0.5, (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM + 1.0 + 0.5, (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM + 1.0 + 0.5, app->draw_color);
			al_restore_state(&old_state);
			break;
		}
		case TOOL_CIRCLE:
		{
			float rx, ry, cx, cy;
			al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
			al_set_target_bitmap(app->scratch_bitmap);
			al_identity_transform(&identity);
			al_use_transform(&identity);
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
			rx = ((t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM - app->peg_x) / 2.0;
			ry = ((t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM - app->peg_y) / 2.0;
			if(fabs(rx) > 0.5 && fabs(ry) > 0.5)
			{
				cx = app->peg_x + rx + 0.5;
				cy = app->peg_y + ry + 0.5;
				al_draw_ellipse(cx, cy, fabs(rx) + 0.25, fabs(ry) + 0.25, app->draw_color, 1.0);
			}
			else
			{
				al_draw_rectangle(app->peg_x + 0.5, app->peg_y + 0.5, (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM + 0.5, (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM + 0.5, app->draw_color, 1.0);
			}
			al_restore_state(&old_state);
			break;
		}
		case TOOL_FILLED_CIRCLE:
		{
			float rx, ry, cx, cy;
			al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
			al_set_target_bitmap(app->scratch_bitmap);
			al_identity_transform(&identity);
			al_use_transform(&identity);
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
			rx = ((t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM - app->peg_x) / 2.0;
			ry = ((t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM - app->peg_y) / 2.0;
			if(fabs(rx) > 0.5 && fabs(ry) > 0.5)
			{
				cx = app->peg_x + rx + 0.5;
				cy = app->peg_y + ry + 0.5;
				al_draw_filled_ellipse(cx, cy, fabs(rx) + 0.25, fabs(ry) + 0.25, app->draw_color);
			}
			else
			{
				al_draw_filled_rectangle(app->peg_x, app->peg_y, (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM + 1.0, (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM + 1.0, app->draw_color);
			}
			al_restore_state(&old_state);
			break;
		}
		case TOOL_CLEAR:
		{
			if(!app->tool_blocked)
			{
				al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
				al_set_target_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0]);
				al_identity_transform(&identity);
				al_use_transform(&identity);
				al_clear_to_color(app->draw_color);
				al_set_target_bitmap(app->scratch_bitmap);
				al_identity_transform(&identity);
				al_use_transform(&identity);
				al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
				al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
				al_draw_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0], 0, 0, 0);
				al_restore_state(&old_state);
				app->tool_blocked = true;
			}
			break;
		}
		case TOOL_FLOOD_FILL:
		{
			if(!app->tool_blocked)
			{
				al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
				flood_fill(app->avatar->image[app->selected_image]->bitmaps->bitmap[0], (int)(t3f_mouse_x - UI_OFFSET_X_EDITOR) / 5, (int)(t3f_mouse_y - UI_OFFSET_Y_EDITOR) / 5, app->draw_color);
				al_set_target_bitmap(app->scratch_bitmap);
				al_identity_transform(&identity);
				al_use_transform(&identity);
				al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
				al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
				al_draw_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0], 0, 0, 0);
				al_restore_state(&old_state);
				app->tool_blocked = true;
			}
			break;
		}
		case TOOL_SELECT:
		{
			al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
			al_set_target_bitmap(app->scratch_bitmap);
			al_identity_transform(&identity);
			al_use_transform(&identity);
			al_restore_state(&old_state);
			app->selection_left = app->peg_x;
			app->selection_top = app->peg_y;
			app->selection_right = (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM;
			app->selection_bottom = (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM;
			if(app->selection_left == app->selection_right || app->selection_top == app->selection_bottom)
			{
				app->selection_active = false;
			}
			else
			{
				if(app->selection_right < app->selection_left)
				{
					val = app->selection_left;
					app->selection_left = app->selection_right;
					app->selection_right = val;
				}
				if(app->selection_bottom < app->selection_top)
				{
					val = app->selection_top;
					app->selection_top = app->selection_bottom;
					app->selection_bottom = val;
				}
				app->float_left = app->selection_left;
				app->float_top = app->selection_top;
				app->float_right = app->selection_right;
				app->float_bottom = app->selection_bottom;
				app->selection_active = true;
			}
			break;
		}
	}
	if(!t3f_mouse_button[0] && !t3f_mouse_button[1])
	{
		al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
		al_set_target_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0]);
		al_identity_transform(&identity);
		al_use_transform(&identity);
		al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
		al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
		al_draw_bitmap(app->scratch_bitmap, 0, 0, 0);
		al_restore_state(&old_state);
		app->state = STATE_UI;
	}
}

void app_float_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	int ox, oy;
	float mx, my;

	mx = (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM;
	my = (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM;
	ox = mx - app->peg_x;
	oy = my - app->peg_y;
	app->float_left += ox;
	app->float_right += ox;
	app->float_top += oy;
	app->float_bottom += oy;
	app->peg_x += ox;
	app->peg_y += oy;

	if(!t3f_mouse_button[0])
	{
		app->state = STATE_UI;
	}
}

/* main logic routine */
void app_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_STATE old_state;
	ALLEGRO_TRANSFORM identity;

	/* copy currently selected image to scratch */
	if(app->avatar)
	{
		al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM | ALLEGRO_STATE_BLENDER);
		al_set_target_bitmap(app->scratch_bitmap);
		al_identity_transform(&identity);
		al_use_transform(&identity);
		al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
		if(app->float_bitmap)
		{
			al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
			al_draw_bitmap(app->float_original, 0, 0, 0);
			al_draw_filled_rectangle(app->selection_left, app->selection_top, app->selection_right, app->selection_bottom, al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
			al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
			al_draw_bitmap(app->float_bitmap, app->float_left, app->float_top, 0);
		}
		else
		{
			al_draw_bitmap(app->avatar->image[app->selected_image]->bitmaps->bitmap[0], 0, 0, 0);
		}
		al_restore_state(&old_state);
	}

	switch(app->state)
	{
		case STATE_UI:
		{
			app_ui_logic(data);
			break;
		}
		case STATE_DRAW:
		{
			app_draw_logic(data);
			break;
		}
		case STATE_FLOAT:
		{
			app_float_logic(data);
			break;
		}
	}
}

static void render_grid(ALLEGRO_BITMAP * grid_bp, float x, float y, int target_w, int target_h)
{
	int i, j, w, h;

	w = al_get_bitmap_width(grid_bp);
	h = al_get_bitmap_height(grid_bp);
	t3f_set_clipping_rectangle(x, y, target_w, target_h);
	al_hold_bitmap_drawing(true);
	for(i = 0; i < target_h / h + 1; i++)
	{
		for(j = 0; j < target_w / w + 1; j++)
		{
			al_draw_bitmap(grid_bp, x + j * w, y + i * h, 0);
		}
	}
	al_hold_bitmap_drawing(false);
	t3f_set_clipping_rectangle(0, 0, 0, 0);
}

/* main rendering routine */
void app_render(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_STATE old_state;
	ALLEGRO_TRANSFORM identity;
	ALLEGRO_BITMAP * bp;
	int i, x, y;

	al_clear_to_color(t3f_color_black);
	render_grid(app->bg_bitmap, UI_OFFSET_X_EDITOR, UI_OFFSET_Y_EDITOR, 64 * 5, 58 * 5);

	if(app->avatar)
	{
		al_draw_filled_rectangle(app->selected_image * UI_SPACE_PREVIEW, UI_OFFSET_Y_PREVIEW, app->selected_image * UI_SPACE_PREVIEW + UI_SPACE_PREVIEW, UI_OFFSET_Y_PREVIEW + 128, al_map_rgba_f(0.0, 0.25, 0.0, 1.0));
		for(i = 0; i < 4; i++)
		{
			bp = app->avatar->image[i]->bitmaps->bitmap[0];
			if(i == app->selected_image)
			{
				bp = app->scratch_bitmap;
			}
			al_draw_bitmap(bp, UI_SPACE_PREVIEW / 2 + i * UI_SPACE_PREVIEW - al_get_bitmap_width(bp) / 2, UI_OFFSET_Y_PREVIEW, 0);
			t3f_draw_text(app->font, t3f_color_white, UI_SPACE_PREVIEW / 2 + i * UI_SPACE_PREVIEW, UI_OFFSET_Y_PREVIEW + 64, 0, 0, 0, T3F_FONT_ALIGN_CENTER, preview_text[i]);
		}
		al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_BLENDER | ALLEGRO_STATE_TRANSFORM);
		al_set_target_bitmap(app->editor_bitmap);
		al_identity_transform(&identity);
		al_use_transform(&identity);
		al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
		al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
		al_draw_bitmap(app->scratch_bitmap, 0, 0, 0);
		al_restore_state(&old_state);
		t3f_draw_scaled_bitmap(app->editor_bitmap, t3f_color_white, UI_OFFSET_X_EDITOR, UI_OFFSET_Y_EDITOR, 0, al_get_bitmap_width(app->editor_bitmap) * UI_ZOOM, al_get_bitmap_height(app->editor_bitmap) * UI_ZOOM, 0);

		if(app->selection_active)
		{
			t3f_set_clipping_rectangle(UI_OFFSET_X_EDITOR, UI_OFFSET_Y_EDITOR, al_get_bitmap_width(app->editor_bitmap) * UI_ZOOM, al_get_bitmap_height(app->editor_bitmap) * UI_ZOOM);
			al_draw_rectangle(UI_OFFSET_X_EDITOR + (app->float_left * UI_ZOOM) + 0.5, UI_OFFSET_Y_EDITOR + (app->float_top * UI_ZOOM) + 0.5, UI_OFFSET_X_EDITOR + (app->float_right * UI_ZOOM) - 1.0 + 0.5, UI_OFFSET_Y_EDITOR + (app->float_bottom * UI_ZOOM) -1.0 + 0.5, t3f_color_black, 1.0);
			al_draw_rectangle(UI_OFFSET_X_EDITOR + (app->float_left * UI_ZOOM) + 1.0 + 0.5, UI_OFFSET_Y_EDITOR + (app->float_top * UI_ZOOM) + 1.0 + 0.5, UI_OFFSET_X_EDITOR + (app->float_right * UI_ZOOM) - 2.0 + 0.5, UI_OFFSET_Y_EDITOR + (app->float_bottom * UI_ZOOM) -2.0 + 0.5, t3f_color_white, 1.0);
			t3f_set_clipping_rectangle(0, 0, 0, 0);
		}
	}

	/* draw UI elements */
	al_draw_bitmap(app->tool_bitmap[app->selected_tool], UI_OFFSET_X_DRAWING_TOOLS, UI_OFFSET_Y_DRAWING_TOOLS, 0);
	al_draw_bitmap(app->palette_tool_bitmap, UI_OFFSET_X_PALETTE_TOOL, UI_OFFSET_Y_PALETTE_TOOL, 0);
	al_draw_bitmap(app->palette_bitmap, UI_OFFSET_X_PALETTE, UI_OFFSET_Y_PALETTE, 0);
	render_grid(app->bg_bitmap, UI_OFFSET_X_PALETTE_TOOL + 3, UI_OFFSET_Y_PALETTE_TOOL + 3, 30, 26);
	render_grid(app->bg_bitmap, UI_OFFSET_X_PALETTE_TOOL + 3 + 32 + 2, UI_OFFSET_Y_PALETTE_TOOL + 3, 30, 26);
	render_grid(app->bg_bitmap, UI_OFFSET_X_PALETTE_TOOL + 3, UI_OFFSET_Y_PALETTE_TOOL + 32 * 8 + 32 + 3, 64, 26);
	al_draw_filled_rectangle(UI_OFFSET_X_PALETTE_TOOL + 3, UI_OFFSET_Y_PALETTE_TOOL + 3, UI_OFFSET_X_PALETTE_TOOL + 1 + 32, UI_OFFSET_Y_PALETTE_TOOL + 2 + 27, app->selected_color[0]);
	al_draw_filled_rectangle(UI_OFFSET_X_PALETTE_TOOL + 3 + 32 + 2, UI_OFFSET_Y_PALETTE_TOOL + 3, UI_OFFSET_X_PALETTE_TOOL + 2 + 32 + 2 + 31, UI_OFFSET_Y_PALETTE_TOOL + 2 + 27, app->selected_color[1]);
	al_draw_filled_rectangle(UI_OFFSET_X_PALETTE_TOOL + 3, UI_OFFSET_Y_PALETTE_TOOL + 32 * 8 + 32 + 3, UI_OFFSET_X_PALETTE_TOOL + 3 + 64, UI_OFFSET_Y_PALETTE_TOOL + 32 * 8 + 32 + 3 + 26, app->hover_color);
	if(app->avatar)
	{
		bp = app->editor_bitmap;
		x = (t3f_mouse_x - UI_OFFSET_X_EDITOR) / UI_ZOOM;
		y = (t3f_mouse_y - UI_OFFSET_Y_EDITOR) / UI_ZOOM;
		render_grid(app->bg_bitmap, UI_OFFSET_X_CLOSE_UP, UI_OFFSET_Y_CLOSE_UP, 64, 64);
		t3f_set_clipping_rectangle(UI_OFFSET_X_CLOSE_UP, UI_OFFSET_Y_CLOSE_UP, 64, 64);
		al_draw_scaled_bitmap(bp, 0, 0, al_get_bitmap_width(bp), al_get_bitmap_height(bp), UI_OFFSET_X_CLOSE_UP - x * UI_CLOSE_UP_ZOOM + 32 - (UI_CLOSE_UP_ZOOM / 2), UI_OFFSET_Y_CLOSE_UP - y * UI_CLOSE_UP_ZOOM + 32 - (UI_CLOSE_UP_ZOOM / 2), al_get_bitmap_width(bp) * UI_CLOSE_UP_ZOOM, al_get_bitmap_height(bp) * UI_CLOSE_UP_ZOOM, 0);
		al_draw_rectangle(UI_OFFSET_X_CLOSE_UP + 32 - (UI_CLOSE_UP_ZOOM / 2) + 0.5, UI_OFFSET_Y_CLOSE_UP + 32 - (UI_CLOSE_UP_ZOOM / 2) + 0.5, UI_OFFSET_X_CLOSE_UP + 32 + (UI_CLOSE_UP_ZOOM / 2) - 1 + 0.5, UI_OFFSET_Y_CLOSE_UP + 32 + (UI_CLOSE_UP_ZOOM / 2) - 1 + 0.5, t3f_color_black, 1.0);
		al_draw_rectangle(UI_OFFSET_X_CLOSE_UP + 32 - (UI_CLOSE_UP_ZOOM / 2) + 1 + 0.5, UI_OFFSET_Y_CLOSE_UP + 32 - (UI_CLOSE_UP_ZOOM / 2) + 1 + 0.5, UI_OFFSET_X_CLOSE_UP + 32 + (UI_CLOSE_UP_ZOOM / 2) - 2 + 0.5, UI_OFFSET_Y_CLOSE_UP + 32 + (UI_CLOSE_UP_ZOOM / 2) - 2 + 0.5, t3f_color_white, 1.0);
//		al_draw_rectangle(UI_OFFSET_X_EDITOR + (app->float_left * UI_ZOOM) + 1.0 + 0.5, UI_OFFSET_Y_EDITOR + (app->float_top * UI_ZOOM) + 1.0 + 0.5, UI_OFFSET_X_EDITOR + (app->float_right * UI_ZOOM) - 2.0 + 0.5, UI_OFFSET_Y_EDITOR + (app->float_bottom * UI_ZOOM) -2.0 + 0.5, t3f_color_white, 1.0);
		t3f_set_clipping_rectangle(0, 0, 0, 0);
	}
	t3gui_render();
}

int main(int argc, char * argv[])
{
	APP_INSTANCE app;

	if(app_initialize(&app, argc, argv))
	{
		t3f_run();
	}
	else
	{
		printf("Error: could not initialize T3F!\n");
	}
	return 0;
}
