#ifndef APP_MAIN_H
#define APP_MAIN_H

#include "t3f/t3f.h"
#include "t3f/font.h"
#include "core/avatar.h"
#include "t3gui/t3gui.h"

#define VERSION_STRING "vGolf Deluxe Avatar Editor v0.1."

#define UI_ZOOM                   5
#define UI_CLOSE_UP_ZOOM         16
#define UI_OFFSET_X_DRAWING_TOOLS 0
#define UI_OFFSET_Y_DRAWING_TOOLS 0
#define UI_OFFSET_X_EDITOR        0
#define UI_OFFSET_Y_EDITOR        (al_get_bitmap_height(app->tool_bitmap[0]))
#define UI_OFFSET_X_PALETTE_TOOL  (t3f_virtual_display_width - al_get_bitmap_width(app->palette_tool_bitmap))
#define UI_OFFSET_Y_PALETTE_TOOL  (al_get_bitmap_height(app->tool_bitmap[0]))
#define UI_OFFSET_X_PALETTE       (UI_OFFSET_X_PALETTE_TOOL + 3)
#define UI_OFFSET_Y_PALETTE       (UI_OFFSET_Y_PALETTE_TOOL + 32)
#define UI_WIDTH_PREVIEW          (t3f_virtual_display_width - al_get_bitmap_width(app->palette_tool_bitmap))
#define UI_SPACE_PREVIEW          (UI_WIDTH_PREVIEW / 4)
#define UI_OFFSET_Y_PREVIEW       (UI_OFFSET_Y_EDITOR + al_get_bitmap_height(app->avatar->image[0]->bitmaps->bitmap[0]) * UI_ZOOM + 14)
#define UI_OFFSET_X_CLOSE_UP 343
#define UI_OFFSET_Y_CLOSE_UP 358

#define STATE_UI    0
#define STATE_DRAW  1
#define STATE_FLOAT 2

#define TOOL_PAINT            0
#define TOOL_LINE             1
#define TOOL_FLOOD_FILL       2
#define TOOL_RECTANGLE        3
#define TOOL_FILLED_RECTANGLE 4
#define TOOL_CLEAR            5
#define TOOL_CIRCLE           6
#define TOOL_FILLED_CIRCLE    7
#define TOOL_SELECT           8

/* structure to hold all of our app-specific data */
typedef struct
{
	int state;
	ALLEGRO_BITMAP * scratch_bitmap;
	ALLEGRO_BITMAP * editor_bitmap;
	ALLEGRO_BITMAP * clipboard_bitmap;
	ALLEGRO_BITMAP * float_bitmap;
	ALLEGRO_BITMAP * float_original;
	ALLEGRO_BITMAP * float_bp;
	int peg_x, peg_y;
	ALLEGRO_COLOR draw_color;
	int selection_top;
	int selection_left;
	int selection_right;
	int selection_bottom;
	bool selection_active;
	int float_top, float_left, float_right, float_bottom;
	int changes;
	ALLEGRO_PATH * last_open_path;
	ALLEGRO_PATH * last_open_image_path;

	/* UI data */
	ALLEGRO_MENU * file_menu;
	ALLEGRO_MENU * edit_menu;
	ALLEGRO_MENU * tool_menu;
	int tool_menu_id[16];
	ALLEGRO_MENU * image_menu;
	ALLEGRO_MENU * turn_menu;
	ALLEGRO_MENU * shift_menu;
	ALLEGRO_MENU * help_menu;
	ALLEGRO_MENU * menu;
	int selected_tool;
	bool tool_blocked;
	ALLEGRO_COLOR selected_color[2];
	ALLEGRO_COLOR hover_color;
	int selected_image;
	T3F_FONT * font;
	ALLEGRO_BITMAP * bg_bitmap;
	ALLEGRO_BITMAP * tool_bitmap[16];
	ALLEGRO_BITMAP * palette_tool_bitmap;
	ALLEGRO_BITMAP * palette_bitmap;
	T3GUI_THEME * box_theme;
	T3GUI_THEME * button_theme;
	T3GUI_THEME * list_box_theme;
	T3GUI_DIALOG * about_dialog;
	T3GUI_DIALOG * unsaved_dialog;
	T3GUI_DIALOG * tags_dialog;
	char tags_edit_text[3][1024];
	int unsaved_dialog_trigger;
	char copyright_message[256];

	/* project data */
	VGOLF_AVATAR * avatar;

} APP_INSTANCE;

void app_unselect(void * data);
void app_update_window_title(void * data);
void app_event_handler(ALLEGRO_EVENT * event, void * data);
void app_logic(void * data);
void app_render(void * data);

#endif
