#include "t3f/t3f.h"
#include "core/avatar.h"
#include "main.h"
#include "undo.h"

static int undo_states = 0;
static int redo_states = 0;
static char undo_fn[1024] = {0};

static char * get_undo_fn(const char * prefix, int i)
{
	snprintf(undo_fn, 1024, "%s%03d.png", prefix, i);
	return undo_fn;
}

bool undo_available(void * data)
{
	return undo_states > 0;
}

static bool _undo_store(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_PATH * path;
	path = al_clone_path(t3f_temp_path);
	bool ret = false;

	if(path)
	{
		al_set_path_filename(path, get_undo_fn("undo_", undo_states));
		if(vgolf_save_avatar(app->avatar, NULL, al_path_cstr(path, '/')))
		{
			undo_states++;
			ret = true;
			app->changes++;
			app_update_window_title(data);
		}
		al_destroy_path(path);
	}
	return ret;
}

void undo_reset(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	undo_states = 0;
	app->changes = 0;
}

bool undo_store(void * data)
{
	if(_undo_store(data))
	{
		redo_reset(data);
		return true;
	}
	return false;
}

bool undo_restore(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_PATH * path;
	VGOLF_AVATAR * avatar = NULL;
	bool ret = false;

	if(undo_states > 0)
	{
		path = al_clone_path(t3f_temp_path);
		if(path)
		{
			al_set_path_filename(path, get_undo_fn("undo_", undo_states - 1));
			avatar = vgolf_load_avatar(al_path_cstr(path, '/'), NULL, NULL);
			if(avatar)
			{
				redo_store(data);
				vgolf_destroy_avatar(app->avatar);
				app->avatar = avatar;
				undo_states--;
				ret = true;
				app->changes--;
				app_update_window_title(data);
			}
			al_destroy_path(path);
		}
	}
	return ret;
}

bool redo_available(void * data)
{
	return redo_states > 0;
}

void redo_reset(void * data)
{
	redo_states = 0;
}

bool redo_store(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_PATH * path;
	path = al_clone_path(t3f_temp_path);
	bool ret = false;

	if(path)
	{
		al_set_path_filename(path, get_undo_fn("redo_", redo_states));
		if(vgolf_save_avatar(app->avatar, NULL, al_path_cstr(path, '/')))
		{
			redo_states++;
			ret = true;
		}
		al_destroy_path(path);
	}
	return ret;
}

bool redo_restore(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_PATH * path;
	VGOLF_AVATAR * avatar = NULL;
	bool ret = false;

	if(redo_states > 0)
	{
		path = al_clone_path(t3f_temp_path);
		if(path)
		{
			al_set_path_filename(path, get_undo_fn("redo_", redo_states - 1));
			avatar = vgolf_load_avatar(al_path_cstr(path, '/'), NULL, NULL);
			if(avatar)
			{
				_undo_store(data);
				vgolf_destroy_avatar(app->avatar);
				app->avatar = avatar;
				redo_states--;
				ret = true;
				app_update_window_title(data);
			}
			al_destroy_path(path);
		}
	}
	return ret;
}
