#include "t3f/t3f.h"
#include "t3f/resource.h"
#include "avatar.h"
#include "resource.h"

void vgolf_build_avatar_from_source(VGOLF_AVATAR * ap)
{
	int i, w, h;

	w = al_get_bitmap_width(ap->source_image->bitmaps->bitmap[0]) / 4;
	h = al_get_bitmap_height(ap->source_image->bitmaps->bitmap[0]);
	for(i = 0; i < VGOLF_AVATAR_MAX_IMAGES; i++)
	{
		ap->image[i] = t3f_create_animation();
		if(ap->image[i])
		{
			ap->image[i]->bitmaps->bitmap[0] = al_create_sub_bitmap(ap->source_image->bitmaps->bitmap[0], i * w, 0, w, h);
			ap->image[i]->bitmaps->count = 1;
			t3f_animation_add_frame(ap->image[i], 0, 0, 0, 0, w, h, 0, 1, 0);
			ap->image[i]->frame[0]->flags = 0;
		}
	}
}

void vgolf_rebuild_avatar_from_source(VGOLF_AVATAR * ap)
{
	int i;

	for(i = 0; i < VGOLF_AVATAR_MAX_IMAGES; i++)
	{
		t3f_destroy_animation(ap->image[i]);
	}
	vgolf_build_avatar_from_source(ap);
}

VGOLF_AVATAR * vgolf_load_avatar(const char * fn, void * palette, const ALLEGRO_FILE_INTERFACE * fi)
{
	VGOLF_AVATAR * ap = NULL;
	ALLEGRO_PATH * pp = NULL;
	const char * ext;
	const ALLEGRO_FILE_INTERFACE * old_fi;
	ALLEGRO_CONFIG * cp;
	const char * val;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}

	ap = malloc(sizeof(VGOLF_AVATAR));
	if(!ap)
	{
		goto cleanup;
	}
	ap->source_image = t3f_create_animation();
	if(!ap->source_image)
	{
		free(ap);
		ap = NULL;
		goto cleanup;
	}

	/* load the source image */
	pp = al_create_path(fn);
	if(!pp)
	{
		goto cleanup;
	}
	ext = al_get_path_extension(pp);
	if(!strcasecmp(ext, ".pcx"))
	{
		t3f_load_resource((void **)&(ap->source_image->bitmaps->bitmap[0]), VGOLF_RESOURCE_TYPE_PCX, fn, 0, 0, 0);
	}
	else
	{
		t3f_load_resource((void **)&(ap->source_image->bitmaps->bitmap[0]), T3F_RESOURCE_TYPE_BITMAP, fn, 0, 0, 0);
	}
	if(!ap->source_image->bitmaps->bitmap[0])
	{
		t3f_destroy_animation(ap->source_image);
		free(ap);
		ap = NULL;
		goto cleanup;
	}
	ap->source_image->bitmaps->count = 1;

	if(palette)
	{
//		convert_palette(bp, pal, palette);
	}
	vgolf_build_avatar_from_source(ap);

	/* load tags */
	al_set_path_extension(pp, ".ini");
	cp = al_load_config_file(al_path_cstr(pp, '/'));
	if(cp)
	{
		val = al_get_config_value(cp, "Tags", "name");
		if(val)
		{
			strncpy(ap->name, val, 256);
		}
		val = al_get_config_value(cp, "Tags", "author");
		if(val)
		{
			strncpy(ap->author, val, 256);
		}
		val = al_get_config_value(cp, "Tags", "comment");
		if(val)
		{
			strncpy(ap->comment, val, 1024);
		}
		al_destroy_config(cp);
	}

	cleanup:
	{
		if(pp)
		{
			al_destroy_path(pp);
		}
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
	}
	return ap;
}

int vgolf_save_avatar(VGOLF_AVATAR * ap, void * pal, const char * fn)
{
	ALLEGRO_STATE old_state;
	ALLEGRO_TRANSFORM identity;
	ALLEGRO_BITMAP * bp;
	int i;
	ALLEGRO_CONFIG * cp;
	ALLEGRO_PATH * pp;

	bp = al_create_bitmap(al_get_bitmap_width(ap->image[0]->bitmaps->bitmap[0]) * 4, al_get_bitmap_height(ap->image[0]->bitmaps->bitmap[0]));
	if(!bp)
	{
		return 0;
	}
	al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM);
	al_identity_transform(&identity);
	al_use_transform(&identity);
	al_set_target_bitmap(bp);
	al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
	for(i = 0; i < 4; i++)
	{
		if(ap->image[i])
		{
			al_draw_bitmap(ap->image[i]->bitmaps->bitmap[0], i * al_get_bitmap_width(ap->image[0]->bitmaps->bitmap[0]), 0, 0);
		}
	}
	al_save_bitmap(fn, bp);
	al_destroy_bitmap(bp);

	/* save tags */
	pp = al_create_path(fn);
	if(pp)
	{
		al_set_path_extension(pp, ".ini");
		cp = al_create_config();
		if(cp)
		{
			al_set_config_value(cp, "Tags", "name", ap->name);
			al_set_config_value(cp, "Tags", "author", ap->author);
			al_set_config_value(cp, "Tags", "comment", ap->comment);
			al_save_config_file(al_path_cstr(pp, '/'), cp);
			al_destroy_config(cp);
		}
		al_destroy_path(pp);
	}

	return 1;
}

void vgolf_destroy_avatar(VGOLF_AVATAR * ap)
{
	int i;

	for(i = 0; i < VGOLF_AVATAR_MAX_IMAGES; i++)
	{
		if(ap->image[i])
		{
			t3f_destroy_animation(ap->image[i]);
		}
	}
	if(ap->source_image)
	{
		t3f_destroy_animation(ap->source_image);
	}
	free(ap);
}
