#include "t3f/t3f.h"
#include "t3f/debug.h"
#include "file.h"
#include "packfile.h"
#include "commentary.h"

static int vgolf_read_commentary_info(ALLEGRO_FILE * fp, VGOLF_COMMENTARY * cp)
{
	char header[16];

	al_fread(fp, header, 16);
	if(strcasecmp(header, "vGolf_vc"))
	{
		return 0;
	}
	al_fread(fp, cp->name, 128);
	al_fread(fp, cp->author, 256);
	al_fread(fp, cp->comment, 1024);
	return header[15];
}

VGOLF_COMMENTARY * load_vgolf_commentary(const char * fn, const ALLEGRO_FILE_INTERFACE * fi)
{
	VGOLF_COMMENTARY * cp = NULL;
	ALLEGRO_FILE * fp = NULL;
	int i, j, m, v;
	const ALLEGRO_FILE_INTERFACE * old_fi;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}
	fp = vgolf_fopen_packfile(fn, "rp");
	if(!fp)
	{
		goto cleanup;
	}

	cp = malloc(sizeof(VGOLF_COMMENTARY));
	if(!cp)
	{
		goto cleanup;
	}

	v = vgolf_read_commentary_info(fp, cp);
	if(!v)
	{
		free(cp);
		cp = NULL;
		goto cleanup;
	}

	switch(v)
	{
		case 1:
		{
			m = al_fread32le(fp);
			for(i = 0; i < m; i++)
			{
				cp->comments[i].count = al_fread32le(fp);
				for(j = 0; j < cp->comments[i].count; j++)
				{
					if(al_fgetc(fp))
					{
						cp->comments[i].comment[j].voice = al_load_sample_f(fp, ".wav");
					}
				}
			}
			break;
		}
	}

	cleanup:
	{
		if(fp)
		{
			al_fclose(fp);
		}
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
	}

	return cp;
}

VGOLF_COMMENTARY * load_vgolf_commentary_info(const char * fn, const ALLEGRO_FILE_INTERFACE * fi)
{
	VGOLF_COMMENTARY * cp = NULL;
	ALLEGRO_FILE * fp = NULL;
	int i, v;
	const ALLEGRO_FILE_INTERFACE * old_fi;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}

	fp = vgolf_fopen_packfile(fn, "rp");
	if(!fp)
	{
		goto cleanup;
	}

	cp = malloc(sizeof(VGOLF_COMMENTARY));
	if(!cp)
	{
		goto cleanup;
	}

	v = vgolf_read_commentary_info(fp, cp);
	if(!v)
	{
		free(cp);
		cp = NULL;
		goto cleanup;
	}

	for(i = 0; i < MAX_MESSAGE_STRING_TYPES; i++)
	{
		cp->comments[i].count = 0;
	}

	cleanup:
	{
		if(fp)
		{
			al_fclose(fp);
		}
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
	}

	return cp;
}

int save_vgolf_commentary(VGOLF_COMMENTARY * cp, const char * fn)
{
	ALLEGRO_FILE * fp;
	char header[16] = "vGolf_vc\0";
	int i, j;

	header[15] = VGOLF_COMMENTARY_VERSION;
	fp = vgolf_fopen_packfile(fn, "wp");
	if(!fp)
	{
		return 0;
	}

	/* save commentary data */
	al_fwrite(fp, header, 16);
	al_fwrite(fp, cp->name, 128);
	al_fwrite(fp, cp->author, 256);
	al_fwrite(fp, cp->comment, 1024);
	al_fwrite32le(fp, MAX_MESSAGE_STRING_TYPES);

	for(i = 0; i < MAX_MESSAGE_STRING_TYPES; i++)
	{
		al_fwrite32le(fp, cp->comments[i].count);
		for(j = 0; j < cp->comments[i].count; j++)
		{
			if(cp->comments[i].comment[j].voice)
			{
				al_fputc(fp, 1);
				al_save_sample_f(fp, ".wav", cp->comments[i].comment[j].voice);
			}
			else
			{
				al_fputc(fp, 0);
			}
		}
	}
	al_fclose(fp);

	return 1;
}

void destroy_vgolf_commentary(VGOLF_COMMENTARY * cp)
{
	int i, j;

	for(i = 0; i < MAX_MESSAGE_STRING_TYPES; i++)
	{
		for(j = 0; j < cp->comments[i].count; j++)
		{
			if(cp->comments[i].comment[j].voice)
			{
				al_destroy_sample(cp->comments[i].comment[j].voice);
			}
		}
	}
	free(cp);
}
