#include "t3f/t3f.h"
#include "t3f/resource.h"
#include <stdio.h>
//#include "defines.h"
#include "course.h"

#include "file.h"
#include "packfile.h"
#include "pcx.h"
#include "utility.h"
#include "resource.h"

void (*vgolf_course_load_callback)(ALLEGRO_COLOR color, const char * text, const char * progress, const char * space) = NULL;

int vgolf_hole_editor_mode = 0;

static void vgolf_conditional_destroy_animation(T3F_ANIMATION ** ap)
{
	if(*ap)
	{
		t3f_destroy_animation(*ap);
		*ap = NULL;
	}
}

static ALLEGRO_BITMAP * load_png_resource_cond(void ** bp, ALLEGRO_FILE * fp, const char * fn)
{
	if(al_fgetc(fp))
	{
		return t3f_load_resource_f(bp, T3F_RESOURCE_TYPE_BITMAP, fp, fn, 0, 0);
	}
	return NULL;
}

void vgolf_set_course_load_callback(void (*callback)(ALLEGRO_COLOR color, const char * text, const char * progress, const char * space))
{
	vgolf_course_load_callback = callback;
}

void vgolf_course_show_load_screen(const char * text)
{
	if(vgolf_course_load_callback)
	{
		vgolf_course_load_callback(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), text, "", "");
	}
}

int vgolf_load_hole_data_fp(HOLE * hole, ALLEGRO_FILE * in_file)
{
	int i, j, k;

	for(i = 0; i < 128; i++) hole->name[i] = al_fgetc(in_file);
	hole->par_score = al_fread16le(in_file);
	hole->body_layers = al_fgetc(in_file);
	for(k = 0; k < hole->body_layers; k++)
	{
		hole->bodies[k] = al_fread16le(in_file);
		if(hole->bodies[k] > MAX_HOLE_POLYS)
		{
			printf("poly out of bounds\n");
			return 0;
		}

		for(i = 0; i < hole->bodies[k]; i++)
		{
			hole->body[k][i].poly.num_vertices = al_fread16le(in_file);
			if(hole->body[k][i].poly.num_vertices > MAX_OBJECT_VERTICES)
			{
				printf("vertex out of bounds\n");
				return 0;
			}
			hole->body[k][i].poly.color = al_fread16le(in_file);
			for(j = 0; j < hole->body[k][i].poly.num_vertices; j++)
			{
				hole->body[k][i].poly.p[j].x = al_fread16le(in_file);
				if(al_fgetc(in_file)) hole->body[k][i].poly.p[j].x *= -1;
				hole->body[k][i].poly.p[j].y = al_fread16le(in_file);
				if(al_fgetc(in_file)) hole->body[k][i].poly.p[j].y *= -1;
//				if(j > 0)
//				{ //do some correction so we don't get vertical lines..
//					if(hole->body[k][i].poly.p[j-1].x == hole->body[k][i].poly.p[j].x)
//					{
//						hole->body[k][i].poly.p[j].x += 0.5;
//					}
//				}
			}
			hole->body[k][i].friction = 0;
			hole->body[k][i].restitution = hole->body[0][i].mass = hole->body[0][i].inv_mass = 0;
			hole->body[k][i].density = 1;
			hole->body[k][i].pos = hole->s;
			hole->body[k][i].vel.x = hole->body[0][i].vel.y = 0;
			hole->body[k][i].glue = 0.0;
			hole->body[k][i].moveable = 0;
		}
	}
	if(hole->body_layers <= 0)
	{
		hole->body_layers = MAX_HOLE_LAYERS;
		hole->bodies[0] = 0;
		hole->bodies[1] = 0;
		hole->bodies[2] = 0;
	}

	hole->num_verts = al_fread16le(in_file);
	for(i = 0; i < hole->num_verts; i++)
	{
		hole->vert[i].s.x = al_fread16le(in_file);
		if(al_fgetc(in_file)) hole->vert[i].s.x *= -1;
		hole->vert[i].s.y = al_fread16le(in_file);
		if(al_fgetc(in_file)) hole->vert[i].s.y *= -1;
		hole->vert[i].flag = al_fread16le(in_file);
		hole->vert[i].radius = al_fread16le(in_file);
		hole->vert[i].num_refs = al_fread16le(in_file);
		for(j = 0; j < hole->vert[i].num_refs; j++)
		{
			hole->vert[i].ref_no[j] = al_fread16le(in_file);
		}
		hole->vert[i].layer = al_fgetc(in_file);
		hole->vert[i].tick = 0;
	}

	return 1;
}

int vgolf_load_hole_fp(HOLE * hole, ALLEGRO_FILE * in_file)
{
	int i, version;
	char temp_char[4];

	//read header..
	al_fread(in_file, temp_char, 4);
	if(temp_char[0] != 'C') return 0;
	if(temp_char[1] != 'R') return 0;
	if(temp_char[2] != 'S') return 0;
	if(temp_char[3] < 37) return 0;

	version = temp_char[3];

	memset(hole, 0, sizeof(HOLE));
	hole->s.x = 320;
	hole->s.y = 240;
	hole->deccel_factor = 0.05f;

	vgolf_load_hole_data_fp(hole, in_file);

	for(i = 0; i < MAX_HOLE_LAYERS; i++)
	{
		if(al_fgetc(in_file))
		{
			hole->layer_pic[i] = t3f_load_bitmap_f(in_file);
		}
		else
		{
			hole->layer_pic[i] = NULL;
		}
	}

	return 1;
}

int vgolf_load_new_hole_fp(HOLE * hole, ALLEGRO_FILE * in_file)
{
	int i, version;
	char temp_char[4];

	//read header..
	al_fread(in_file, temp_char, 4);
	if(temp_char[0] != 'C') return 0;
	if(temp_char[1] != 'R') return 0;
	if(temp_char[2] != 'S') return 0;
	if(temp_char[3] < 37) return 0;

	version = temp_char[3];

	memset(hole, 0, sizeof(HOLE));
	hole->s.x = 320;
	hole->s.y = 240;
	hole->deccel_factor = 0.05f;

	vgolf_load_hole_data_fp(hole, in_file);

	/* read graphics offsets for layers */
	for(i = 0; i < MAX_HOLE_LAYERS; i++)
	{
		hole->layer_pic_offset[0][i] = al_fread32le(in_file);
		hole->layer_pic_offset[1][i] = al_fread32le(in_file);
	}

	return 1;
}

int vgolf_load_hole_graphics(HOLE * hole, const char * fn, int type, const ALLEGRO_FILE_INTERFACE * fi)
{
	ALLEGRO_FILE * gfp;
	ALLEGRO_PATH * pp;
	const ALLEGRO_FILE_INTERFACE * old_fi;
	const char * type_ext[2] = {".gsd", ".ghd"};
	int j;
	int ret = 0;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}
	pp = al_create_path(fn);
	if(pp)
	{
		al_set_path_extension(pp, type_ext[type]);
		gfp = al_fopen(al_path_cstr(pp, '/'), "rb");
		if(!gfp)
		{
			type = (type + 1) % 2;
			al_set_path_extension(pp, type_ext[type]);
			gfp = al_fopen(al_path_cstr(pp, '/'), "rb");
		}
		if(gfp)
		{
			for(j = 0; j < MAX_HOLE_LAYERS; j++)
			{
				al_fseek(gfp, hole->layer_pic_offset[type][j], ALLEGRO_SEEK_SET);
				hole->layer_pic[j] = load_png_resource_cond((void **)&hole->layer_pic[j], gfp, al_path_cstr(pp, '/'));
			}
			al_fclose(gfp);
			ret = 1;
		}
	}
	if(fi)
	{
		al_set_new_file_interface(old_fi);
	}
	return ret;
}

void vgolf_destroy_hole_graphics(HOLE * hole)
{
	int j;

	for(j = 0; j < MAX_HOLE_LAYERS; j++)
	{
		if(hole->layer_pic[j])
		{
			t3f_destroy_resource(hole->layer_pic[j]);
			hole->layer_pic[j] = NULL;
		}
	}
}

int vgolf_load_hole(HOLE * hole, char * filename)
{
	ALLEGRO_FILE * in_file;

	in_file = al_fopen(filename, "rb");
	if(!in_file)
	{
		return 0;
	}
	vgolf_load_hole_fp(hole, in_file);
	al_fclose(in_file);
	return 1;
}

/* free hole image data */
void vgolf_free_hole(HOLE * hole)
{
	int i;

	for(i = 0; i < MAX_HOLE_LAYERS; i++)
	{
		if(hole->layer_pic[i])
		{
			t3f_destroy_resource(hole->layer_pic[i]);
			hole->layer_pic[i] = NULL;
		}
	}
}

int vgolf_save_hole_fp(HOLE * hole, ALLEGRO_FILE * out_file)
{
	int i, j, k;

	//write header..
	al_fputc(out_file, 'C');
	al_fputc(out_file, 'R');
	al_fputc(out_file, 'S');
	al_fputc(out_file, 37);

	for(i = 0; i < 128; i++) al_fputc(out_file, hole->name[i]);
	al_fwrite16le(out_file, hole->par_score);
	al_fputc(out_file, hole->body_layers);
	for(k = 0; k < hole->body_layers; k++)
	{
		al_fwrite16le(out_file, hole->bodies[k]);
		for(i = 0; i < hole->bodies[k]; i++)
		{
			al_fwrite16le(out_file, hole->body[k][i].poly.num_vertices);
			al_fwrite16le(out_file, hole->body[k][i].poly.color);
			for(j = 0; j < hole->body[k][i].poly.num_vertices; j++)
			{
				al_fwrite16le(out_file, abs((int)hole->body[k][i].poly.p[j].x));
				al_fputc(out_file, (hole->body[k][i].poly.p[j].x < 0));
				al_fwrite16le(out_file, abs((int)hole->body[k][i].poly.p[j].y));
				al_fputc(out_file, (hole->body[k][i].poly.p[j].y < 0));
			}
		}
	}

	al_fwrite16le(out_file, hole->num_verts);
	for(i = 0; i < hole->num_verts; i++)
	{
		al_fwrite16le(out_file, abs((int)hole->vert[i].s.x));
		al_fputc(out_file, (hole->vert[i].s.x < 0));
		al_fwrite16le(out_file, abs((int)hole->vert[i].s.y));
		al_fputc(out_file, (hole->vert[i].s.y < 0));
		al_fwrite16le(out_file, hole->vert[i].flag);
		al_fwrite16le(out_file, hole->vert[i].radius);
		al_fwrite16le(out_file, hole->vert[i].num_refs);
		for(j = 0; j < hole->vert[i].num_refs; j++)
		{
			al_fwrite16le(out_file, hole->vert[i].ref_no[j]);
		}
		al_fputc(out_file, hole->vert[i].layer);
	}

	/* write graphics offsets */
	for(i = 0; i < MAX_HOLE_LAYERS; i++)
	{
		al_fwrite32le(out_file, hole->layer_pic_offset[0][i]);
		al_fwrite32le(out_file, hole->layer_pic_offset[1][i]);
	}
	return 1;
}

int vgolf_save_hole(HOLE * hole, char * filename)
{
        ALLEGRO_FILE * out_file;

        out_file = al_fopen(filename, "wb");
        if(!out_file)
        {
	        return 0;
        }
        vgolf_save_hole_fp(hole, out_file);
        al_fclose(out_file);

        return 1;
}

static int save_png_cond(ALLEGRO_FILE * fp, ALLEGRO_BITMAP * bp)
{
	if(bp)
	{
		al_fputc(fp, 1);
		if(!t3f_save_bitmap_f(fp, bp))
		{
			printf("save failed\n");
		}
	}
	else
	{
		al_fputc(fp, 0);
	}
	return 1;
}

static int save_ani_cond(ALLEGRO_FILE * fp, T3F_ANIMATION * ap)
{
	if(ap)
	{
		al_fputc(fp, 1);
		if(!t3f_save_animation_f(ap, fp))
		{
			printf("save failed\n");
		}
	}
	else
	{
		al_fputc(fp, 0);
		printf("no animation\n");
	}
	return 1;
}

/*static T3F_ANIMATION * load_animation_from_png_cond(ALLEGRO_FILE * fp)
{
	if(al_fgetc(fp))
	{
		return vgolf_bitmap_to_animation(t3f_load_bitmap_f(fp));
	}
	printf("failed to load image\n");
	return NULL;
} */

static T3F_ANIMATION * load_animation_cond(ALLEGRO_FILE * fp, const char * filename)
{
	if(al_fgetc(fp))
	{
		return t3f_load_animation_f(fp, filename);
	}
	printf("failed to load image\n");
	return NULL;
}

static ALLEGRO_BITMAP * load_png_cond(ALLEGRO_FILE * fp)
{
	if(al_fgetc(fp))
	{
		return t3f_load_bitmap_f(fp);
	}
	printf("failed to load image\n");
	return NULL;
}

void vgolf_read_new_course_info(VGOLF_COURSE * cr, ALLEGRO_FILE * fp, int version, const char * filename, int type)
{
	ALLEGRO_PATH * pp;
	ALLEGRO_FILE * gfp;
	char * type_ext[2] = {".gsd", ".ghd"};

	/* course info */
	al_fread(fp, cr->name, 128);
	al_fread(fp, cr->author, 256);
	al_fread(fp, cr->comment, 1024);
	cr->num_holes = al_fread32le(fp);
	cr->course_par = al_fread32le(fp);
	cr->icon_offset[0] = al_fread32le(fp);
	cr->icon_offset[1] = al_fread32le(fp);
	cr->victory_graphics_offset[0] = al_fread32le(fp);
	cr->victory_graphics_offset[1] = al_fread32le(fp);
	pp = al_create_path(filename);
	if(pp)
	{
		al_set_path_extension(pp, type_ext[type]);
		gfp = al_fopen(al_path_cstr(pp, '/'), "rb");
		if(!gfp)
		{
			type = (type + 1)  % 2;
			al_set_path_extension(pp, type_ext[type]);
			gfp = al_fopen(al_path_cstr(pp, '/'), "rb");
		}
		if(gfp)
		{
			al_fseek(gfp, cr->icon_offset[type], ALLEGRO_SEEK_SET);
			cr->image_bank.iconpic = load_animation_cond(gfp, al_path_cstr(pp, '/'));
			al_fclose(gfp);
		}
		al_destroy_path(pp);
	}
}

void vgolf_read_course_info(VGOLF_COURSE * cr, ALLEGRO_FILE * fp, int version, const char * filename)
{
	/* course info */
	al_fread(fp, cr->name, 128);
	al_fread(fp, cr->author, 256);
	al_fread(fp, cr->comment, 1024);
	cr->num_holes = al_fread32le(fp);
	cr->course_par = al_fread32le(fp);
	cr->image_bank.iconpic = load_animation_cond(fp, filename);
}

VGOLF_COURSE * vgolf_load_course_info(const char * filename, int type, const ALLEGRO_FILE_INTERFACE * fi)
{
	VGOLF_COURSE * cr = NULL;
	ALLEGRO_FILE * fp;
	char header[16] = {0};
	bool new = false;
	const ALLEGRO_FILE_INTERFACE * old_fi;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}
	fp = al_fopen(filename, "rb");
	if(!fp)
	{
		printf("failed to open file\n");
		goto cleanup;
	}
	if(al_fread(fp, header, 16) != 16)
	{
		al_fclose(fp);
		goto cleanup;
	}
	if(!strcasecmp(header, "vGolf_crs2"))
	{
		new = true;
	}
	else if(strcasecmp(header, "vGolf_crs"))
	{
		al_fclose(fp);
		goto cleanup;
	}

	cr = malloc(sizeof(VGOLF_COURSE));
	if(!cr)
	{
		al_fclose(fp);
		goto cleanup;
	}
	memset(cr, 0, sizeof(VGOLF_COURSE));
	if(new)
	{
		vgolf_read_new_course_info(cr, fp, header[15], filename, type);
	}
	else
	{
		vgolf_read_course_info(cr, fp, header[15], filename);
	}
	al_fclose(fp);

	cleanup:
	{
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
		return cr;
	}
}

int vgolf_load_course_graphics(VGOLF_COURSE * cr, const char * filename, int type, int flags, const ALLEGRO_FILE_INTERFACE * fi)
{
	ALLEGRO_FILE * fp = NULL;
	ALLEGRO_PATH * pp = NULL;
	ALLEGRO_STATE old_state;
	ALLEGRO_BITMAP * bp;
	char * type_ext[2] = {".gsd", ".ghd"};
	int i;
	int ret = 0;
	const ALLEGRO_FILE_INTERFACE * old_fi;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}
	pp = al_create_path(filename);
	if(!pp)
	{
		goto cleanup;
	}
	al_set_path_extension(pp, type_ext[type]);

	fp = al_fopen(al_path_cstr(pp, '/'), "rb");
	if(!fp)
	{
		type = (type + 1) % 2;
		al_set_path_extension(pp, type_ext[type]);
		fp = al_fopen(al_path_cstr(pp, '/'), "rb");
		if(!fp)
		{
			goto cleanup;
		}
	}

	/* load palette (some objects reference colors by number so we need to keep a palette) */
	al_store_state(&old_state, ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
	bp = load_png_cond(fp);
	if(bp)
	{
		for(i = 0; i < 256; i++)
		{
			al_unmap_rgb(al_get_pixel(bp, i, 0), &cr->palette.entry[i].r, &cr->palette.entry[i].g, &cr->palette.entry[i].b);
		}
		al_destroy_bitmap(bp);
	}
	al_restore_state(&old_state);

	/* course data */
	//tee
	cr->image_bank.vert_pic[0][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	//hole
	cr->image_bank.vert_pic[1][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.vert_pic[1][1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	//trees
	cr->image_bank.object_pic[0][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.object_pic[1][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	//laser/wall
	cr->image_bank.object_pic[2][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.object_pic[2][1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	//teleport pad
	cr->image_bank.object_pic[3][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.object_pic[3][1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	//conveyor belt
	cr->image_bank.object_pic[4][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.object_pic[4][1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.object_pic[4][2] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.object_pic[4][3] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	//animals
	cr->image_bank.animal_pic[0][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.animal_pic[0][1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.animal_pic[0][2] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.animal_pic[0][3] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.animal_pic[1][0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.animal_pic[1][1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.animal_pic[1][2] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.animal_pic[1][3] = load_animation_cond(fp, al_path_cstr(pp, '/'));

	//load scoreboard images
	cr->scoreboard.bgpic = load_animation_cond(fp, al_path_cstr(pp, '/'));

	ret = 1;

	cleanup:
	{
		if(fp)
		{
			al_fclose(fp);
		}
		if(pp)
		{
			al_destroy_path(pp);
		}
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
	}
	return ret;
}

void vgolf_destroy_course_graphics(VGOLF_COURSE * cr)
{
	//tee
	vgolf_conditional_destroy_animation(&cr->image_bank.vert_pic[0][0]);
	//hole
	vgolf_conditional_destroy_animation(&cr->image_bank.vert_pic[1][0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.vert_pic[1][1]);
	//trees
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[0][0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[1][0]);
	//laser/wall
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[2][0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[2][1]);
	//teleport pad
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[3][0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[3][1]);
	//conveyor belt
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[4][0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[4][1]);
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[4][2]);
	vgolf_conditional_destroy_animation(&cr->image_bank.object_pic[4][3]);

	//animals
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[0][0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[0][1]);
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[0][2]);
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[0][3]);
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[1][0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[1][1]);
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[1][2]);
	vgolf_conditional_destroy_animation(&cr->image_bank.animal_pic[1][3]);
	/* Don't destroy this animation since we need it for the victory screen
	 * vgolf_conditional_destroy_animation(&cr->scoreboard.bgpic); */
}

int vgolf_load_course_victory_graphics(VGOLF_COURSE * cr, const char * filename, int type, int flags, const ALLEGRO_FILE_INTERFACE * fi)
{
	ALLEGRO_FILE * fp = NULL;
	ALLEGRO_PATH * pp = NULL;
	char * type_ext[2] = {".gsd", ".ghd"};
	const ALLEGRO_FILE_INTERFACE * old_fi;
	int ret = 0;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}
	pp = al_create_path(filename);
	if(!pp)
	{
		goto cleanup;
	}
	al_set_path_extension(pp, type_ext[type]);

	fp = al_fopen(al_path_cstr(pp, '/'), "rb");
	if(!fp)
	{
		type = (type + 1) % 2;
		al_set_path_extension(pp, type_ext[type]);
		fp = al_fopen(al_path_cstr(pp, '/'), "rb");
		if(!fp)
		{
			goto cleanup;
		}
	}

	al_fseek(fp, cr->victory_graphics_offset[type], ALLEGRO_SEEK_SET);
	cr->image_bank.victory_bgpic = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.victory_podiumpic = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.victory_cloudpic[0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.victory_cloudpic[1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.victory_trophypic[0] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.victory_trophypic[1] = load_animation_cond(fp, al_path_cstr(pp, '/'));
	cr->image_bank.victory_trophypic[2] = load_animation_cond(fp, al_path_cstr(pp, '/'));

	ret = 1;

	cleanup:
	{
		if(fp)
		{
			al_fclose(fp);
		}
		if(pp)
		{
			al_destroy_path(pp);
		}
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
	}

	return ret;
}

void vgolf_destroy_course_victory_graphics(VGOLF_COURSE * cr)
{
	vgolf_conditional_destroy_animation(&cr->image_bank.victory_bgpic);
	vgolf_conditional_destroy_animation(&cr->image_bank.victory_podiumpic);
	vgolf_conditional_destroy_animation(&cr->image_bank.victory_cloudpic[0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.victory_cloudpic[1]);
	vgolf_conditional_destroy_animation(&cr->image_bank.victory_trophypic[0]);
	vgolf_conditional_destroy_animation(&cr->image_bank.victory_trophypic[1]);
	vgolf_conditional_destroy_animation(&cr->image_bank.victory_trophypic[2]);
}

VGOLF_COURSE * vgolf_load_new_course(const char * filename, int type, int flags, const ALLEGRO_FILE_INTERFACE * fi)
{
	char * type_ext[2] = {".gsd", ".ghd"};
	VGOLF_COURSE * cr = NULL;
	ALLEGRO_FILE * fp = NULL;
	ALLEGRO_FILE * gfp = NULL;
	ALLEGRO_PATH * pp = NULL;
	char header[16] = {0};
	int i, j;
	const ALLEGRO_FILE_INTERFACE * old_fi;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}
	fp = al_fopen(filename, "rb");
	if(!fp)
	{
		goto cleanup;
	}
	al_fread(fp, header, 16);
	if(strcasecmp(header, "vGolf_crs2"))
	{
		goto cleanup;
	}

	cr = malloc(sizeof(VGOLF_COURSE));
	if(!cr)
	{
		goto cleanup;
	}
	memset(cr, 0, sizeof(VGOLF_COURSE));

	vgolf_course_show_load_screen("Loading");
	vgolf_read_new_course_info(cr, fp, header[15], filename, type);
	vgolf_course_show_load_screen("Loading");

	if(!vgolf_load_course_graphics(cr, filename, type, flags, fi))
	{
		vgolf_destroy_course(cr);
		cr = NULL;
		goto cleanup;
	}

	for(i = 0; i < cr->num_holes; i++)
	{
		vgolf_course_show_load_screen("Loading");
		vgolf_load_new_hole_fp(&cr->hole[i], fp);
	}

	/* optionally load victory graphics */
	if(flags & VGOLF_COURSE_LOAD_ALL)
	{
		vgolf_load_course_victory_graphics(cr, filename, type, flags, fi);
	}

	/* optionally load all hole graphics */
	if(flags & VGOLF_COURSE_LOAD_ALL)
	{
		pp = al_create_path(filename);
		if(pp)
		{
			al_set_path_extension(pp, type_ext[type]);
			gfp = al_fopen(al_path_cstr(pp, '/'), "rb");
			if(!gfp)
			{
				type = (type + 1) % 2;
				al_set_path_extension(pp, type_ext[type]);
				gfp = al_fopen(al_path_cstr(pp, '/'), "rb");
			}
			if(gfp)
			{
				for(i = 0; i < cr->num_holes; i++)
				{
					for(j = 0; j < MAX_HOLE_LAYERS; j++)
					{
						vgolf_course_show_load_screen("Loading");
						al_fseek(gfp, cr->hole[i].layer_pic_offset[type][j], ALLEGRO_SEEK_SET);
						cr->hole[i].layer_pic[j] = load_png_resource_cond((void **)&cr->hole[i].layer_pic[j], gfp, al_path_cstr(pp, '/'));
					}
				}
			}
		}
	}

	//width and height of the numbers on the scoreboard
	cr->scoreboard.num_w = 17;
	cr->scoreboard.num_h = 17;
	//x,y values of the scoreboard for positioning on the screen
	cr->scoreboard.x = 0;
	cr->scoreboard.y = 0;
	//x,y offset values of the course name on the scoreboard
	cr->scoreboard.cname_x = 20;
	cr->scoreboard.cname_y = 15;
	//x,y offset values of the menu items on the scoreboard
	cr->scoreboard.menu_x[0] = 130;
	cr->scoreboard.menu_y[0] = 440;
	cr->scoreboard.menu_x[1] = 420;
	cr->scoreboard.menu_y[1] = 440;
	//x,y offset values of the grids on the scoreboard (player 1-4)
	cr->scoreboard.grid_x[0] = 20 + 30;
	cr->scoreboard.grid_y[0] = 50 + 8;
	cr->scoreboard.grid_x[1] = 20 + 320 + 10;
	cr->scoreboard.grid_y[1] = 50 + 8;
	cr->scoreboard.grid_x[2] = 20 + 30;
	cr->scoreboard.grid_y[2] = 250;
	cr->scoreboard.grid_x[3] = 20 + 320 + 10;
	cr->scoreboard.grid_y[3] = 250;
	//# of segments on scoreboard and # of active ones
	cr->scoreboard.num_scoreboard_segs = 10;
	cr->scoreboard.scoreboard_seg_count = 0;
	cr->scoreboard.scoreboard_seg_counter = 0;
	cr->scoreboard.scoreboard_seg_counter_end = 48;

	cr->legacy = false;

	cleanup:
	{
		if(fp)
		{
			al_fclose(fp);
		}
		if(gfp)
		{
			al_fclose(gfp);
		}
		if(pp)
		{
			al_destroy_path(pp);
		}
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
	}

	return cr;
}

VGOLF_COURSE * vgolf_load_course(const char * filename, int type, int flags, const ALLEGRO_FILE_INTERFACE * fi)
{
	VGOLF_COURSE * cr = NULL;
	ALLEGRO_FILE * fp = NULL;
	ALLEGRO_BITMAP * bp;
	ALLEGRO_STATE old_state;
	char header[16] = {0};
	int i;
	const ALLEGRO_FILE_INTERFACE * old_fi;

	if(fi)
	{
		old_fi = al_get_new_file_interface();
		al_set_new_file_interface(fi);
	}

	fp = al_fopen(filename, "rb");
	if(!fp)
	{
		goto cleanup;
	}
	al_fread(fp, header, 16);
	if(!strcasecmp(header, "vGolf_crs2"))
	{
		al_fclose(fp);
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
		return vgolf_load_new_course(filename, type, flags, fi);
	}
	else if(strcasecmp(header, "vGolf_crs"))
	{
		goto cleanup;
	}

	cr = malloc(sizeof(VGOLF_COURSE));
	if(!cr)
	{
		goto cleanup;
	}
	memset(cr, 0, sizeof(VGOLF_COURSE));

	vgolf_course_show_load_screen("Loading");
	vgolf_read_course_info(cr, fp, header[15], filename);
	vgolf_course_show_load_screen("Loading");

	/* load palette (some objects reference colors by number so we need to keep a palette) */
	al_store_state(&old_state, ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
	if(flags & VGOLF_COURSE_LOAD_EDIT)
	{
		al_set_new_bitmap_flags(ALLEGRO_NO_PREMULTIPLIED_ALPHA | ALLEGRO_MIN_LINEAR | ALLEGRO_MAG_LINEAR);
	}

	bp = load_png_cond(fp);
	if(bp)
	{
		for(i = 0; i < 256; i++)
		{
			al_unmap_rgb(al_get_pixel(bp, i, 0), &cr->palette.entry[i].r, &cr->palette.entry[i].g, &cr->palette.entry[i].b);
		}
		al_destroy_bitmap(bp);
	}
	al_restore_state(&old_state);

	/* course data */
	//tee
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.vert_pic[0][0] = load_animation_cond(fp, filename);
	//hole
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.vert_pic[1][0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.vert_pic[1][1] = load_animation_cond(fp, filename);
	//trees
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[0][0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[1][0] = load_animation_cond(fp, filename);
	//laser/wall
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[2][0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[2][1] = load_animation_cond(fp, filename);
	//teleport pad
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[3][0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[3][1] = load_animation_cond(fp, filename);
	//conveyor belt
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[4][0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[4][1] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[4][2] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.object_pic[4][3] = load_animation_cond(fp, filename);
	//animals
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[0][0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[0][1] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[0][2] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[0][3] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[1][0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[1][1] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[1][2] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.animal_pic[1][3] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.ball_pic = t3f_load_animation("data/graphics/ball.t3a");
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.ball_marker_pic = t3f_load_animation("data/graphics/marker.t3a");
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.target_pic = t3f_load_animation("data/graphics/target.t3a");
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.club_pic = t3f_load_animation("data/graphics/club.t3a");
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.club_outline_pic = t3f_load_animation("data/graphics/club_outline.t3a");

	//load victory screen images
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.victory_bgpic = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.victory_podiumpic = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.victory_cloudpic[0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.victory_cloudpic[1] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.victory_trophypic[0] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.victory_trophypic[1] = load_animation_cond(fp, filename);
	vgolf_course_show_load_screen("Loading");
	cr->image_bank.victory_trophypic[2] = load_animation_cond(fp, filename);
	//load scoreboard images
	vgolf_course_show_load_screen("Loading");
	cr->scoreboard.bgpic = load_animation_cond(fp, filename);

	for(i = 0; i < cr->num_holes; i++)
	{
		vgolf_course_show_load_screen("Loading");
		vgolf_load_hole_fp(&cr->hole[i], fp);
	}

	//width and height of the numbers on the scoreboard
	cr->scoreboard.num_w = 17;
	cr->scoreboard.num_h = 17;
	//x,y values of the scoreboard for positioning on the screen
	cr->scoreboard.x = 0;
	cr->scoreboard.y = 0;
	//x,y offset values of the course name on the scoreboard
	cr->scoreboard.cname_x = 20;
	cr->scoreboard.cname_y = 15;
	//x,y offset values of the menu items on the scoreboard
	cr->scoreboard.menu_x[0] = 130;
	cr->scoreboard.menu_y[0] = 440;
	cr->scoreboard.menu_x[1] = 420;
	cr->scoreboard.menu_y[1] = 440;
	//x,y offset values of the grids on the scoreboard (player 1-4)
	cr->scoreboard.grid_x[0] = 20 + 30;
	cr->scoreboard.grid_y[0] = 50 + 8;
	cr->scoreboard.grid_x[1] = 20 + 320 + 10;
	cr->scoreboard.grid_y[1] = 50 + 8;
	cr->scoreboard.grid_x[2] = 20 + 30;
	cr->scoreboard.grid_y[2] = 250;
	cr->scoreboard.grid_x[3] = 20 + 320 + 10;
	cr->scoreboard.grid_y[3] = 250;
	//# of segments on scoreboard and # of active ones
	cr->scoreboard.num_scoreboard_segs = 10;
	cr->scoreboard.scoreboard_seg_count = 0;
	cr->scoreboard.scoreboard_seg_counter = 0;
	cr->scoreboard.scoreboard_seg_counter_end = 48;

	cleanup:
	{
		if(fp)
		{
			al_fclose(fp);
		}
		if(fi)
		{
			al_set_new_file_interface(old_fi);
		}
	}

	return cr;
}

void vgolf_write_course_header(ALLEGRO_FILE * fp)
{
	char header[16] = "vGolf_crs2";
	header[15] = 2;

	al_fwrite(fp, header, 16);
}

void vgolf_write_course_info(VGOLF_COURSE * cr, ALLEGRO_FILE * fp)
{
	al_fwrite(fp, cr->name, 128);
	al_fwrite(fp, cr->author, 256);
	al_fwrite(fp, cr->comment, 1024);
	al_fwrite32le(fp, cr->num_holes);
	al_fwrite32le(fp, cr->course_par);
	al_fwrite32le(fp, cr->icon_offset[0]);
	al_fwrite32le(fp, cr->icon_offset[1]);
	al_fwrite32le(fp, cr->victory_graphics_offset[0]);
	al_fwrite32le(fp, cr->victory_graphics_offset[1]);
}

int vgolf_save_course_info(VGOLF_COURSE * cr, const char * fn)
{
	ALLEGRO_FILE * fp;
	char header[16] = "vGolf_crs";

	fp = al_fopen(fn, "wb");
	if(!fp)
	{
		return 0;
	}
	al_fwrite(fp, header, 16);
	al_fwrite(fp, cr->name, 128);
	al_fwrite(fp, cr->author, 256);
	al_fwrite(fp, cr->comment, 1024);
	al_fwrite32le(fp, cr->num_holes);
	al_fwrite32le(fp, cr->course_par);
	al_fputc(fp, 1);
	t3f_save_animation_f(cr->image_bank.iconpic, fp);
	al_fclose(fp);
	return 1;
}

int vgolf_save_course_graphics(VGOLF_COURSE * cr, const char * filename, int type)
{
	ALLEGRO_FILE * fp;
	ALLEGRO_PATH * pp;
	ALLEGRO_STATE old_state;
	ALLEGRO_BITMAP * bp;
	char * type_ext[2] = {".gsd", ".ghd"};
	int i, j;

	pp = al_create_path(filename);
	if(!pp)
	{
		return 0;
	}
	al_set_path_extension(pp, type_ext[type]);

	fp = al_fopen(al_path_cstr(pp, '/'), "wb");
	if(!fp)
	{
		al_destroy_path(pp);
		return 0;
	}

	/* save palette (some objects reference colors by number so we need to keep a palette) */
	al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
	bp = al_create_bitmap(256, 1);
	if(bp)
	{
		al_set_target_bitmap(bp);
		for(i = 0; i < 256; i++)
		{
			al_put_pixel(i, 0, al_map_rgb(cr->palette.entry[i].r, cr->palette.entry[i].g, cr->palette.entry[i].b));
		}
		save_png_cond(fp, bp);
		al_destroy_bitmap(bp);
	}
	else
	{
		printf("no palette\n");
		save_png_cond(fp, NULL);
	}
	al_restore_state(&old_state);

	/* save main course graphics for objects, etc. */
	//tee
	save_ani_cond(fp, cr->image_bank.vert_pic[0][0]);
	//hole
	save_ani_cond(fp, cr->image_bank.vert_pic[1][0]);
	save_ani_cond(fp, cr->image_bank.vert_pic[1][1]);
	//trees
	save_ani_cond(fp, cr->image_bank.object_pic[0][0]);
	save_ani_cond(fp, cr->image_bank.object_pic[1][0]);
	//laser/wall
	save_ani_cond(fp, cr->image_bank.object_pic[2][0]);
	save_ani_cond(fp, cr->image_bank.object_pic[2][1]);
	//teleport pad
	save_ani_cond(fp, cr->image_bank.object_pic[3][0]);
	save_ani_cond(fp, cr->image_bank.object_pic[3][1]);
	//conveyor belt
	save_ani_cond(fp, cr->image_bank.object_pic[4][0]);
	save_ani_cond(fp, cr->image_bank.object_pic[4][1]);
	save_ani_cond(fp, cr->image_bank.object_pic[4][2]);
	save_ani_cond(fp, cr->image_bank.object_pic[4][3]);
	//animals
	save_ani_cond(fp, cr->image_bank.animal_pic[0][0]);
	save_ani_cond(fp, cr->image_bank.animal_pic[0][1]);
	save_ani_cond(fp, cr->image_bank.animal_pic[0][2]);
	save_ani_cond(fp, cr->image_bank.animal_pic[0][3]);
	save_ani_cond(fp, cr->image_bank.animal_pic[1][0]);
	save_ani_cond(fp, cr->image_bank.animal_pic[1][1]);
	save_ani_cond(fp, cr->image_bank.animal_pic[1][2]);
	save_ani_cond(fp, cr->image_bank.animal_pic[1][3]);
	//scoreboard images
	save_ani_cond(fp, cr->scoreboard.bgpic);

	//save victory screen images (store offset so we can get back here later) */
	cr->victory_graphics_offset[type] = al_ftell(fp);
	save_ani_cond(fp, cr->image_bank.victory_bgpic);
	save_ani_cond(fp, cr->image_bank.victory_podiumpic);
	save_ani_cond(fp, cr->image_bank.victory_cloudpic[0]);
	save_ani_cond(fp, cr->image_bank.victory_cloudpic[1]);
	save_ani_cond(fp, cr->image_bank.victory_trophypic[0]);
	save_ani_cond(fp, cr->image_bank.victory_trophypic[1]);
	save_ani_cond(fp, cr->image_bank.victory_trophypic[2]);

	cr->icon_offset[type] = al_ftell(fp);
	save_ani_cond(fp, cr->image_bank.iconpic);

	/* save hole layer images */
	for(j = 0; j < cr->num_holes; j++)
	{
		//save hole bitmap overlays (if present..)
		for(i = 0; i < MAX_HOLE_LAYERS; i++)
		{
			cr->hole[j].layer_pic_offset[type][i] = al_ftell(fp);
			if(cr->hole[j].layer_pic[i] == NULL)
			{
				al_fputc(fp, 0);
			}
			else
			{
				al_fputc(fp, 1);
				t3f_save_bitmap_f(fp, cr->hole[j].layer_pic[i]);
			}
		}
	}
	al_fclose(fp);
	return 1;
}

int vgolf_save_course(VGOLF_COURSE * cr, const char * filename, int type)
{
	ALLEGRO_FILE * fp;
	int i;

	/* save graphics first so we get hole graphics offsets */
	if(!vgolf_save_course_graphics(cr, filename, type))
	{
		return 0;
	}

	fp = al_fopen(filename, "wb");
	if(!fp)
	{
		return 0;
	}
	vgolf_write_course_header(fp);

	/* course info */
	vgolf_write_course_info(cr, fp);

	/* course data */
	for(i = 0; i < cr->num_holes; i++)
	{
		vgolf_save_hole_fp(&cr->hole[i], fp);
	}
	al_fclose(fp);

	return 1;
}

void vgolf_destroy_course(VGOLF_COURSE * cr)
{
	int i;

	if(!cr)
	{
		return;
	}
	if(cr->scoreboard.bgpic)
	{
		t3f_destroy_animation(cr->scoreboard.bgpic);
	}
	vgolf_destroy_course_graphics(cr);
	vgolf_destroy_course_victory_graphics(cr);
	t3f_destroy_animation(cr->image_bank.iconpic);
	for(i = 0; i < cr->num_holes; i++)
	{
		vgolf_free_hole(&cr->hole[i]);
	}
	free(cr);
}

int vgolf_course_make_color(VGOLF_COURSE * cr, int r, int g, int b)
{
	int i;

	for(i = 0; i < 256; i++)
	{
		if(cr->palette.entry[i].r == r && cr->palette.entry[i].g == g && cr->palette.entry[i].b == b)
		{
			return i;
		}
	}
	return 0;
}

ALLEGRO_COLOR vgolf_course_get_color(VGOLF_COURSE * cr, int i)
{
	return al_map_rgb(cr->palette.entry[i].r, cr->palette.entry[i].g, cr->palette.entry[i].b);
}
