#include "t3f/t3f.h"
#include "t3f/menu.h"
#ifdef ALLEGRO_WINDOWS
	#include "t3f/windows.h"
#endif

#include "core/commentary.h"
#include "core/resource.h"
#include "t3gui/t3gui.h"

#include "main.h"
#include "ui_proc.h"

void app_event_handler(ALLEGRO_EVENT * event, void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	T3GUI_ELEMENT * ep = (T3GUI_ELEMENT *)event->user.data2;

	switch(event->type)
	{
		case T3GUI_EVENT_CLOSE:
		{
			if(ep == app->tags_ok_element)
			{
				strncpy(app->commentary->name, app->tags_title, 128);
				strncpy(app->commentary->author, app->tags_author, 256);
				strncpy(app->commentary->comment, app->tags_comment, 1024);
			}
			break;
		}
		case T3GUI_EVENT_CANCEL:
		{
			break;
		}
	}
	t3f_event_handler(event);
}

/* main logic routine */
void app_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	t3gui_logic();
	switch(app->state)
	{
		case STATE_EDIT_MAIN:
		{
			if(app->type_list_element->d1 != app->last_index)
			{
				app->voice_list_element->d1 = 0;
			}
			if(app->last_index != app->type_list_element->d1)
			{
				app->last_index = app->type_list_element->d1;
				ui_update_buttons(app);
			}
			break;
		}
		case STATE_EDIT_TAGS:
		{
			break;
		}
	}
}

/* main rendering routine */
void app_render(void * data)
{
	al_clear_to_color(t3f_color_black);
	t3gui_render();
}

bool app_setup_menus(APP_INSTANCE * app)
{
	app->file_menu = al_create_menu();
	if(!app->file_menu)
	{
		return false;
	}
	t3f_add_menu_item(app->file_menu, "&New", 0, NULL, ui_menu_proc_file_new, NULL);
	t3f_add_menu_item(app->file_menu, "&Open", 0, NULL, ui_menu_proc_file_open, NULL);
	t3f_add_menu_item(app->file_menu, "&Save", ALLEGRO_MENU_ITEM_DISABLED, NULL, ui_menu_proc_file_save, ui_menu_update_item);
	t3f_add_menu_item(app->file_menu, "Save &As", ALLEGRO_MENU_ITEM_DISABLED, NULL, ui_menu_proc_file_save_as, ui_menu_update_item);
	t3f_add_menu_item(app->file_menu, NULL, 0, NULL, NULL, NULL);
	t3f_add_menu_item(app->file_menu, "&Tags", ALLEGRO_MENU_ITEM_DISABLED, NULL, ui_menu_proc_file_tags, ui_menu_update_item);
	#ifndef ALLEGRO_MACOSX
		t3f_add_menu_item(app->file_menu, NULL, 0, NULL, NULL, NULL);
		t3f_add_menu_item(app->file_menu, "&Exit", 0, NULL, ui_menu_proc_file_exit, NULL);
	#endif

	app->help_menu = al_create_menu();
	if(!app->help_menu)
	{
		return false;
	}
//	t3f_add_menu_item(app->help_menu, "&Manual", 0, NULL, ui_menu_proc_help_manual, NULL);
	t3f_add_menu_item(app->help_menu, "&About", 0, NULL, ui_menu_proc_help_about, NULL);

	app->menu = al_create_menu();
	if(!app->menu)
	{
		return false;
	}
	t3f_add_menu_item(app->menu, "&File", 0, app->file_menu, NULL, NULL);
	t3f_add_menu_item(app->menu, "&Help", 0, app->help_menu, NULL, NULL);
	return true;
}

bool app_setup_dialogs(APP_INSTANCE * app)
{
	T3GUI_ELEMENT * dp;
	int text_width;
	int bezel = 8;
	int font_height;
	int w;

	/* main dialog */
	app->dialog[0] = t3gui_create_dialog();
	t3gui_dialog_add_element(app->dialog[0], app->button_theme, t3gui_box_proc, 0, 0, 640, 480, 0, 0, 0, 0, NULL, NULL, NULL);
	if(!app->dialog[0])
	{
		return false;
	}
	app->type_list_element = t3gui_dialog_add_element(app->dialog[0], app->text_box_theme, t3gui_list_proc, 16, 16, 320 - 32, 480 - 32 - 48, 0, 0, 0, 0, ui_type_list_proc, NULL, app);
	if(!app->type_list_element)
	{
		return false;
	}
	app->voice_list_element = t3gui_dialog_add_element(app->dialog[0], app->text_box_theme, t3gui_list_proc, 320 + 16, 16, 320 - 32, 480 - 32 - 48, 0, 0, 0, 0, ui_voice_list_proc, NULL, app);
	w = (640 - 32) / 4;
	if(!app->voice_list_element)
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[0], app->button_theme, t3gui_push_button_proc, 16 + w * 0, 480 - 32 - 16, w, 32, 0, D_DISABLED, 0, 0, "Add", ui_voice_add_button_proc, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[0], app->button_theme, t3gui_push_button_proc, 16 + w * 1, 480 - 32 - 16, w, 32, 0, D_DISABLED, 0, 0, "Delete", ui_voice_delete_button_proc, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[0], app->button_theme, t3gui_push_button_proc, 16 + w * 2, 480 - 32 - 16, w, 32, 0, D_DISABLED, 0, 0, "Export", ui_voice_export_button_proc, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[0], app->button_theme, t3gui_push_button_proc, 16 + w * 3, 480 - 32 - 16, w, 32, 0, D_DISABLED, 0, 0, "Listen", ui_voice_listen_button_proc, NULL))
	{
		return false;
	}

	/* tags dialog */
	app->dialog[1] = t3gui_create_dialog();
	if(!app->dialog[1])
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[1], app->button_theme, t3gui_box_proc, 160 - 16, 120 - 16, 320 + 32, 360 + 27, 0, 0, 0, 0, NULL, NULL, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[1], app->text_box_theme, t3gui_text_proc, 160, 120, 320, 20, 0, 0, ALLEGRO_ALIGN_LEFT, 0, "Title", NULL, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[1], app->text_box_theme, t3gui_edit_proc, 160, 120 + 22 * 1, 320, 20, 0, 0, 256, 0, app->tags_title, NULL, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[1], app->text_box_theme, t3gui_text_proc, 160, 120 + 22 * 3, 320, 16, 0, 0, ALLEGRO_ALIGN_LEFT, 0, "Author", NULL, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[1], app->text_box_theme, t3gui_edit_proc, 160, 120 + 22 * 4, 320, 20, 0, 0, 256, 0, app->tags_author, NULL, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[1], app->text_box_theme, t3gui_text_proc, 160, 120 + 22 * 6, 320, 16, 0, 0, ALLEGRO_ALIGN_LEFT, 0, "Comment", NULL, NULL))
	{
		return false;
	}
	if(!t3gui_dialog_add_element(app->dialog[1], app->text_box_theme, t3gui_editbox_proc, 160, 120 + 22 * 7, 320, 147, 0, 0, 1024, 0, app->tags_comment, NULL, NULL))
	{
		return false;
	}
	app->tags_ok_element = t3gui_dialog_add_element(app->dialog[1], app->button_theme, t3gui_button_proc, 160, 120 - 16 + 360 - 25, 160, 32, 0, D_EXIT, ALLEGRO_ALIGN_LEFT, 0, "Okay", NULL, NULL);
	dp = t3gui_dialog_add_element(app->dialog[1], app->button_theme, t3gui_button_proc, 320, 120 - 16 + 360 - 25, 160, 32, 0, D_EXIT, ALLEGRO_ALIGN_LEFT, 0, "Cancel", NULL, NULL);
	if(!app->tags_ok_element)
	{
		return false;
	}
	t3gui_centre_dialog(app->dialog[1]->element, 640, 480);

	font_height = al_get_font_line_height(app->about_button_theme->state[0].font[0]);
	text_width = al_get_text_width(app->about_button_theme->state[0].font[0], T3F_APP_TITLE " v" T3F_APP_VERSION);
	app->about_dialog = t3gui_create_dialog();
	if(!app->about_dialog)
	{
		return false;
	}
	t3gui_dialog_add_element(app->about_dialog, app->about_box_theme, t3gui_box_proc, 0, 0, bezel + text_width + bezel, bezel + font_height * 2 + bezel + 32 + bezel, 0, 0, 0, 0, NULL, NULL, NULL);
	t3gui_dialog_add_element(app->about_dialog, app->about_button_theme, t3gui_text_proc, bezel, bezel, text_width, font_height, 0, 0, ALLEGRO_ALIGN_CENTRE, 0, T3F_APP_TITLE " v" T3F_APP_VERSION, NULL, NULL);
	t3gui_dialog_add_element(app->about_dialog, app->about_button_theme, t3gui_text_proc, bezel, bezel + font_height, text_width, font_height, 0, 0, ALLEGRO_ALIGN_CENTRE, 0, app->copyright_message, NULL, NULL);
	t3gui_dialog_add_element(app->about_dialog, app->about_button_theme, t3gui_push_button_proc, bezel + text_width / 2 - 32, bezel + font_height * 2 + bezel, 64, 32, 0, D_EXIT, 0, 0, "Okay", NULL, NULL);
	t3gui_center_dialog(app->about_dialog, 640, 480);

	return true;
}

/* initialize our app, load graphics, etc. */
bool app_initialize(APP_INSTANCE * app, int argc, char * argv[])
{
	/* initialize T3F */
	if(!t3f_initialize(T3F_APP_TITLE, 640, 480, 60.0, app_logic, app_render, T3F_DEFAULT | T3F_USE_MENU, app))
	{
		printf("Error initializing T3F\n");
		return false;
	}
	t3f_set_event_handler(app_event_handler);
	memset(app, 0, sizeof(APP_INSTANCE));
	#ifdef ALLEGRO_WINDOWS
		t3f_windows_text_to_utf8(T3F_APP_COPYRIGHT, app->copyright_message, 256);
	#else
		strcpy(app->copyright_message, T3F_APP_COPYRIGHT);
	#endif
	ui_load_config();

	if(!t3gui_init())
	{
		printf("Error initializing T3GUI\n");
		return false;
	}

	app->button_theme = t3gui_load_theme("data/button_theme.ini", 0);
	if(!app->button_theme)
	{
		return false;
	}
	app->text_box_theme = t3gui_load_theme("data/listbox_theme.ini", 0);
	if(!app->text_box_theme)
	{
		return false;
	}
	app->about_box_theme = t3gui_load_theme("data/about_box_theme.ini", 0);
	if(!app->about_box_theme)
	{
		return false;
	}
	app->about_button_theme = t3gui_load_theme("data/about_button_theme.ini", 0);
	if(!app->about_button_theme)
	{
		return false;
	}

	if(!app_setup_menus(app))
	{
		return false;
	}
	if(!app_setup_dialogs(app))
	{
		return false;
	}

	t3f_attach_menu(app->menu);

	vgolf_register_resource_loaders();
	t3gui_show_dialog(app->dialog[0], t3f_queue, T3GUI_PLAYER_NO_ESCAPE, app);
	app->state = STATE_EDIT_MAIN;
	return true;
}

int main(int argc, char * argv[])
{
	APP_INSTANCE app;
	int i;

	if(app_initialize(&app, argc, argv))
	{
		t3f_run();
	}
	else
	{
		printf("Error: could not initialize T3F!\n");
	}

	for(i = 0; i < 2; i++)
	{
		t3gui_close_dialog(app.dialog[i]);
	}
	t3gui_exit();
	return 0;
}
