#ifndef VGOLF_COURSE_H
#define VGOLF_COURSE_H

#include "t3f/t3f.h"
#include "t3f/animation.h"
#include "poly.h"
#include "body.h"
#include "pcx.h"

#define MAX_COURSE_HOLES     18

#define MAX_HOLE_POLYS       32
#define MAX_HOLE_LAYERS       3
#define MAX_HOLE_VERTS       64

#define MAX_VERT_FLAG_TYPES 255
#define MAX_VERT_RADIUS    1000
#define MAX_VERT_REFS         8
#define MAX_VERT_REF_TYPES   20

#define MAX_COURSE_VERT_PIC_TYPES 16
#define MAX_VERT_ANIMS             4
#define MAX_COURSE_ANIMAL_TYPES    4
#define MAX_ANIMAL_ANIMS           4
#define MAX_COURSE_OBJECT_TYPES    8
#define MAX_OBJECT_ANIMS           4
#define MAX_COIN_PICS              4

#define VERT_FLAG_TEE               0
#define VERT_FLAG_HOLE              1
#define VERT_FLAG_TREE_0          100
#define VERT_FLAG_TREE_1          101
#define VERT_FLAG_TREE_2          102
#define VERT_FLAG_TREE_3          103
#define VERT_FLAG_ANIMAL_0         50
#define VERT_FLAG_ANIMAL_1         51
#define VERT_FLAG_ANIMAL_2         52
#define VERT_FLAG_HAZARD_WATER     30
#define VERT_FLAG_HAZARD_SAND      31
#define VERT_FLAG_HAZARD_ROUGH     32
#define VERT_FLAG_HAZARD_SLIME     33
#define VERT_FLAG_PIPE_ENTER       40
#define VERT_FLAG_PIPE_EXIT        41
#define VERT_FLAG_INDENT_HILL      20
#define VERT_FLAG_INDENT_RUT       21
#define VERT_FLAG_POLY_MORPH       60
#define VERT_FLAG_POLY_MOVE_HORIZ  61
#define VERT_FLAG_POLY_MOVE_VERT   62
#define VERT_FLAG_POLY_MOTION_AREA 63
#define VERT_FLAG_GOPHER_MOUND     70
#define VERT_FLAG_CLOUD_VERT       80
#define VERT_FLAG_CLOUD_HORIZ      81
#define VERT_FLAG_COIN_0          120
#define VERT_FLAG_LASER_SEND      110
#define VERT_FLAG_LASER_RECV      111
#define VERT_FLAG_LASER_RESPAWN   112
#define VERT_FLAG_TELEPORT        130
#define VERT_FLAG_CONVEYOR        140
#define VERT_FLAG_BLINKER         145
#define VERT_FLAG_POLY_DOOR_HORIZ 150
#define VERT_FLAG_POLY_DOOR_VERT  151
#define VERT_FLAG_SMASH_RESPAWN   152
#define VERT_FLAG_BUBBLE_HORIZ    153
#define VERT_FLAG_BUBBLE_VERT     154
#define VERT_FLAG_LAYER_TRANSFER  155
#define VERT_FLAG_TRANSLUCENCY    156
#define VERT_FLAG_NULL            999

#define VGOLF_COURSE_LOAD_ALL       1 // load all graphics at start
#define VGOLF_COURSE_LOAD_EDIT      2 // intent on saving later

typedef struct
{

	VERTEX s;
	int flag;
	int ref_no[MAX_VERT_REFS];
	float radius;
	int num_refs;
	int layer;
	int tick;

} VERT;

typedef struct
{

	int angle[MAX_OBJECT_VERTICES];

} POLY_ANG;

typedef struct
{

	char name[128];
	int par_score;
	int num_verts;
	int num_coins;
	float deccel_factor;

	BODY body[MAX_HOLE_LAYERS][MAX_HOLE_POLYS];
	int body_layers;
	int bodies[MAX_HOLE_LAYERS];
	VERT vert[MAX_HOLE_VERTS];
	VERTEX s;

	ALLEGRO_BITMAP * layer_pic[MAX_HOLE_LAYERS];
	unsigned long layer_pic_offset[2][MAX_HOLE_LAYERS];

} HOLE;

typedef struct
{

	T3F_ANIMATION * vert_pic[MAX_COURSE_VERT_PIC_TYPES][MAX_VERT_ANIMS];
	T3F_ANIMATION * animal_pic[MAX_COURSE_ANIMAL_TYPES][MAX_ANIMAL_ANIMS];
	T3F_ANIMATION * object_pic[MAX_COURSE_OBJECT_TYPES][MAX_OBJECT_ANIMS];
	T3F_ANIMATION * victory_bgpic;               //victory screen bg
	T3F_ANIMATION * victory_podiumpic;           //victory screen podium
	T3F_ANIMATION * victory_cloudpic[4];         //victory screen floater
	T3F_ANIMATION * victory_trophypic[3];        //victory screen trophies (gold, silver, bronze)
	T3F_ANIMATION * iconpic;              //the hole's icon
	T3F_ANIMATION * ball_pic;
	T3F_ANIMATION * ball_marker_pic;
	T3F_ANIMATION * target_pic;
	T3F_ANIMATION * club_pic;
	T3F_ANIMATION * club_outline_pic;
	ALLEGRO_BITMAP * ball_splash_pic[5];
	ALLEGRO_BITMAP * scorecard;
	T3F_ATLAS * game_atlas;

} COURSE_IMAGE_BANK;

typedef struct
{

	T3F_ANIMATION * bgpic;                   //scoreboard background image
//	ALLEGRO_BITMAP * gridpic;                 //image for displaying the total course strokes thusfar
//	ALLEGRO_BITMAP * numpic[19];              //images of the numbers (0-18)

	int num_w, num_h;                 //width, height of the numbers
	int x,y;                          //screen coordinates for the scoreboard
	int menu_x[2], menu_y[2];         //screen coordinates for scoreboard menu items
	int grid_x[4], grid_y[4];  //screen coordinates for all players' grids
	int cname_x, cname_y;             //screen coordinates for displaying the course name

	short num_scoreboard_segs;        //# of "wood" segments on scoreboard
	short scoreboard_seg_count;       //# of activated segments on scoreboard currently
	short scoreboard_seg_counter;     //counter for having the segments gradually appear onscreen
	short scoreboard_seg_counter_end; //the ending value for the timer (increase for bigger delay)

} SCOREBOARD;

typedef struct
{

	char name[128];                   //course name
	char author[256];
	char comment[1024];
	char fname[1024];                   //course filename
	char fname_prefix[1024];            //course filename prefix
	int num_holes;                    //# holes in this course
	int empty_color;                  //color this course uses for filling empty screen space
	int course_par;                   //total course par score

	COURSE_IMAGE_BANK image_bank;     //bank of all images used for course display

	HOLE current_hole;                //just keep the current hole in memory
	HOLE hole[18];

	SCOREBOARD scoreboard;            //course scoreboard
	PALETTE palette;
	int icon_offset[2];
	int victory_graphics_offset[2];

	bool legacy;

} VGOLF_COURSE;

extern int vgolf_hole_editor_mode;

int vgolf_load_hole_fp(HOLE * hole, ALLEGRO_FILE * in_file);
int vgolf_save_hole_fp(HOLE * hole, ALLEGRO_FILE * out_file);
void vgolf_free_hole(HOLE * hole);

void vgolf_set_course_load_callback(void (*callback)(ALLEGRO_COLOR color, const char * text, const char * progress, const char * space));
VGOLF_COURSE * vgolf_load_course_info(const char * filename, int type, const ALLEGRO_FILE_INTERFACE * fi);
VGOLF_COURSE * vgolf_load_course(const char * filename, int type, int flags, const ALLEGRO_FILE_INTERFACE * fi);
int vgolf_save_course_info(VGOLF_COURSE * cr, const char * fn);
int vgolf_save_course(VGOLF_COURSE * cr, const char * filename, int type);
void vgolf_destroy_course(VGOLF_COURSE * cp);

/* on-the-fly loading functions */
int vgolf_load_course_graphics(VGOLF_COURSE * cr, const char * filename, int type, int flags, const ALLEGRO_FILE_INTERFACE * fi);
void vgolf_destroy_course_graphics(VGOLF_COURSE * cr);
int vgolf_load_course_victory_graphics(VGOLF_COURSE * cr, const char * filename, int type, int flags, const ALLEGRO_FILE_INTERFACE * fi);
void vgolf_destroy_course_victory_graphics(VGOLF_COURSE * cr);
int vgolf_load_hole_graphics(HOLE * hole, const char * fn, int type, const ALLEGRO_FILE_INTERFACE * fi);
void vgolf_destroy_hole_graphics(HOLE * hole);

int vgolf_course_make_color(VGOLF_COURSE * cr, int r, int g, int b);
ALLEGRO_COLOR vgolf_course_get_color(VGOLF_COURSE * cr, int i);

#endif
