#include "t3f/t3f.h"
#include "t3f/view.h"

#include "course.h"
#include "utility.h"

#include "vert.h"

#define VERT_POLY_LINE_THICKNESS (floor(t3f_display_scale_x) / t3f_display_scale_x) * zoom

//do all the initial vert calculations/settings..
void hole_initialize_verts(HOLE * hole)
{
	int v;
	int hole_vert[32];
	int hole_verts = 0;

	for(v = 0; v < hole->num_verts; v++)
	{
		if(hole->vert[v].flag == VERT_FLAG_ANIMAL_0 || hole->vert[v].flag == VERT_FLAG_ANIMAL_1)
		{
			//refs 4 and 5 define a bounding box that holds the animal inside it
			hole->vert[v].ref_no[4] = hole->s.x+hole->vert[v].s.x;
			hole->vert[v].ref_no[5] = hole->s.y+hole->vert[v].s.y;
			hole->vert[v].ref_no[0] = hole->vert[v].ref_no[1] = hole->vert[v].ref_no[2] = hole->vert[v].ref_no[3] = 0;
		}
		else if(hole->vert[v].flag == VERT_FLAG_POLY_MORPH)
		{
			if(hole->vert[v].ref_no[0] == 0)
			{
				//tag the associated poly to be a rotating poly
				hole->body[hole->vert[v].layer][hole->vert[v].ref_no[1]].poly.color = POLY_TYPE_MORPH_ROTATE;
				hole->body[hole->vert[v].layer][hole->vert[v].ref_no[1]].poly.misc = v;
			}
		}
		else if(hole->vert[v].flag == VERT_FLAG_POLY_MOVE_HORIZ)
		{
			//tag the associated poly to be a horizontally moving poly
			hole->body[hole->vert[v].layer][hole->vert[v].ref_no[1]].poly.color = POLY_TYPE_MOTION_HORIZ;
		}
		else if(hole->vert[v].flag == VERT_FLAG_POLY_MOVE_VERT)
		{
			//tag the associated poly to be a vertically moving poly
			hole->body[hole->vert[v].layer][hole->vert[v].ref_no[1]].poly.color = POLY_TYPE_MOTION_VERT;
		}
		else if(hole->vert[v].flag == VERT_FLAG_HOLE)
		{
			hole_vert[hole_verts] = v;
			hole_verts++;
		}
		else if(hole->vert[v].flag == VERT_FLAG_LASER_SEND)
		{
			/* add laser poly */
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].poly.num_vertices = 2;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].poly.color = POLY_TYPE_EMPTY_WALL;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].poly.p[0].x = hole->vert[v].s.x;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].poly.p[0].y = hole->vert[v].s.y;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].poly.p[1].x = hole->vert[hole->vert[v].ref_no[5]].s.x;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].poly.p[1].y = hole->vert[hole->vert[v].ref_no[5]].s.y;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].friction = 0;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].restitution = hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].mass = hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].inv_mass = 0;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].density = 1;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].pos = hole->s;
			hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].vel.x = hole->body[hole->vert[v].layer][hole->bodies[hole->vert[v].layer]].vel.y = 0;
			hole->vert[v].ref_no[4] = hole->bodies[hole->vert[v].layer];
			hole->bodies[hole->vert[v].layer]++;

			//reset laser active and timers
			hole->vert[v].ref_no[0] = hole->vert[v].ref_no[1] = 0;
			//make the associated poly empty while laser is inactive
			hole->body[hole->vert[v].layer][hole->vert[v].ref_no[4]].poly.color = POLY_TYPE_EMPTY_WALL;
		}
		else if(hole->vert[v].flag == VERT_FLAG_TELEPORT)
		{
			//reset animation counter
			hole->vert[v].ref_no[5] = 0;
		}
		else if(hole->vert[v].flag == VERT_FLAG_BUBBLE_HORIZ || hole->vert[v].flag == VERT_FLAG_BUBBLE_VERT)
		{
			hole->vert[v].ref_no[0] = hole->vert[v].ref_no[1] = hole->vert[v].ref_no[2] = hole->vert[v].ref_no[3] = hole->vert[v].ref_no[4] = hole->vert[v].ref_no[5] = 0;
		}
		else if(hole->vert[v].flag == VERT_FLAG_CONVEYOR)
		{
			hole->vert[v].ref_no[1] = 1;
			hole->vert[v].ref_no[4] = 0;
			hole->body[hole->vert[v].layer][hole->vert[v].ref_no[5]].poly.color = POLY_TYPE_CONVEYOR_WALL;
		}
		else if(hole->vert[v].flag == VERT_FLAG_LAYER_TRANSFER)
		{
			hole->body[hole->vert[v].layer][hole->vert[v].ref_no[0]].poly.color = POLY_TYPE_LAYER_TRANSFER;
		}
		else if(hole->vert[v].flag == VERT_FLAG_TRANSLUCENCY)
		{
			hole->body[hole->vert[v].layer][hole->vert[v].ref_no[0]].poly.color = POLY_TYPE_TRANSLUCENCY;
		}
		else if(hole->vert[v].flag == VERT_FLAG_POLY_DOOR_HORIZ)
		{
			//set these polys as walls
			hole->body[hole->vert[v].layer][hole->vert[hole->vert[v].ref_no[4]].ref_no[4]].poly.color = POLY_TYPE_DOOR_L;
			hole->body[hole->vert[v].layer][hole->vert[hole->vert[v].ref_no[4]].ref_no[5]].poly.color = POLY_TYPE_DOOR_R;
		}
		else if(hole->vert[v].flag == VERT_FLAG_POLY_DOOR_VERT)
		{
			//set these polys as walls
			hole->body[hole->vert[v].layer][hole->vert[hole->vert[v].ref_no[4]].ref_no[4]].poly.color = POLY_TYPE_DOOR_U;
			hole->body[hole->vert[v].layer][hole->vert[hole->vert[v].ref_no[4]].ref_no[5]].poly.color = POLY_TYPE_DOOR_D;
		}
	}
}

//handle the logic of the animal verts
void handle_animal_vert(VERT * vert, HOLE * hole)
{
	int i;

	vert->s.x += vert->ref_no[1]*ANIMAL_MOVE_FACTOR;
	vert->s.y += vert->ref_no[2]*ANIMAL_MOVE_FACTOR;

	//keep animal within original radius..
	if(distance(v_offset(hole->s, vert->s).x, v_offset(hole->s, vert->s).y, vert->ref_no[4], vert->ref_no[5]) > vert->radius)
	{
		vert->s.x -= vert->ref_no[1]*ANIMAL_MOVE_FACTOR;
		vert->s.y -= vert->ref_no[2]*ANIMAL_MOVE_FACTOR;
	}

	//animate animal..
	vert->ref_no[3]++;   //+= timer_change
	if(!(vert->ref_no[3] % ANIMAL_ANIM_TIME))
	{
		if(vert->ref_no[1] > 0) vert->ref_no[0] = 0;
		else vert->ref_no[0] = 2;
	}
	if(!(vert->ref_no[3] % (ANIMAL_ANIM_TIME*2)))
	{
		if(vert->ref_no[1] > 0) vert->ref_no[0] = 1;
		else vert->ref_no[0] = 3;
	}

	//give animal random movement vector..
	if(vert->ref_no[3] >= (ANIMAL_MOVE_TIME-50)+vgolf_rand()%(ANIMAL_MOVE_TIME))
	{
		vert->ref_no[1] = vgolf_rand() % 3;
		i = vgolf_rand()%2;
		if(i) vert->ref_no[1] *= -1;

		vert->ref_no[2] = vgolf_rand() % 3;
		i = vgolf_rand()%2;
		if(i) vert->ref_no[2] *= -1;

		vert->ref_no[3] = 0;
	}
}

//handle the logic of the bubble vert
void handle_bubble_vert(VERT * vert, HOLE * hole)
{
	//inactive, set speed
	if(!vert->ref_no[0])
	{
		vert->ref_no[0] = 1;
		vert->ref_no[1] = (vgolf_rand()%4)+2;
		vert->ref_no[2] = 0;
		vert->ref_no[3] = 0;
		vert->ref_no[4] = (vgolf_rand()%16)+12;  //random perpendicular movement
	}

	vert->s.x -= vert->ref_no[1]*CLOUD_MOVE_FACTOR*(vert->flag == VERT_FLAG_BUBBLE_HORIZ);
	vert->s.y -= vert->ref_no[1]*CLOUD_MOVE_FACTOR*(vert->flag == VERT_FLAG_BUBBLE_VERT);

	//handle swaying motion
	if(vert->flag == VERT_FLAG_BUBBLE_HORIZ)
	{
		if(!vert->ref_no[2])
		{
			vert->s.y -= 0.25;
		}
		else
		{
			vert->s.y += 0.25;
		}

		//wrap when edge of screen hit..
		if(hole->s.x+vert->s.x+vert->radius < 0)
		{
			vert->s.x = t3f_virtual_display_width+vert->radius;
			vert->s.y = (vgolf_rand()%(t3f_virtual_display_height/2-(int)vert->radius))*(vgolf_rand()%2 ? -1 : 1);
		}
	}
	else
	{
		if(!vert->ref_no[2])
		{
			vert->s.x -= 0.25;
		}
		else
		{
			vert->s.x += 0.25;
		}

		//wrap when edge of screen hit..
		if(hole->s.y+vert->s.y+vert->radius < 0)
		{
			vert->s.x = (vgolf_rand()%(t3f_virtual_display_width/2-(int)vert->radius))*(vgolf_rand()%2 ? -1 : 1);
			vert->s.y = t3f_virtual_display_height+vert->radius;
		}
	}
	if(++vert->ref_no[3] >= vert->ref_no[4])
	{
		vert->ref_no[3] = 0;
		vert->ref_no[2] = !vert->ref_no[2];
	}
}

//handle the logic of the cloud vert
void handle_cloud_vert(VERT * vert, HOLE * hole)
{

        //inactive, set speed
        if(!vert->ref_no[0]){
                vert->ref_no[0] = 1;
                vert->ref_no[1] = (vgolf_rand()%4)+2;
                vert->ref_no[2] = 0;
                vert->ref_no[3] = 0;
                vert->ref_no[4] = (vgolf_rand()%(int)vert->radius/8);
                }

        vert->s.x += vert->ref_no[1]*CLOUD_MOVE_FACTOR*(vert->flag == VERT_FLAG_CLOUD_HORIZ);
        vert->s.y += vert->ref_no[1]*CLOUD_MOVE_FACTOR*(vert->flag == VERT_FLAG_CLOUD_VERT);

        //wrap when edge of screen hit..
        if(hole->s.x+vert->s.x-vert->radius > t3f_virtual_display_width){
                vert->s.x = -t3f_virtual_display_width/2-vert->radius;
                vert->s.y = (vgolf_rand()%(t3f_virtual_display_height/2-(int)vert->radius))*(vgolf_rand()%2 ? -1 : 1);
                }
        else if(hole->s.y+vert->s.y-vert->radius > t3f_virtual_display_height){
                vert->s.y = t3f_virtual_display_height/2-vert->radius;
                vert->s.x = (vgolf_rand()%(t3f_virtual_display_width/2-(int)vert->radius))*(vgolf_rand()%2 ? -1 : 1);
                }

}

void handle_blinker_vert(VERT * vert, HOLE * hole)
{
	if(!vert->ref_no[0])
	{  //laser inactive, run timer to delay ceiling
		if(++(vert->ref_no[1]) >= vert->ref_no[2])
		{  //keep counting while the laser is being fired (multiply by two to slow it down a little)
			vert->ref_no[1] = 0;     //reset the delay timer
			vert->ref_no[0] = 1;     //reset the activtiy of the laser send and receive verts
		}
	}
	else
	{   //fire laser
		if(++(vert->ref_no[1]) >= vert->ref_no[3])
		{
			vert->ref_no[1] = 0;    //reset the laser delay timer
			vert->ref_no[0] = 0;    //reset the activity of the laser send and receive verts
		}
	}
}

void handle_door_vert(VERT * vert, HOLE * hole)
{
	if(!vert->ref_no[0])
	{  //count delay..
		if(++vert->ref_no[1] >= vert->ref_no[2])
		{
			vert->ref_no[1] = 0;
			vert->ref_no[0] = 1;
			//tag back to door polys
			if(vert->flag == VERT_FLAG_POLY_DOOR_HORIZ)
			{
				hole->body[vert->layer][hole->vert[vert->ref_no[4]].ref_no[4]].poly.color = POLY_TYPE_DOOR_L;
				hole->body[vert->layer][hole->vert[vert->ref_no[4]].ref_no[5]].poly.color = POLY_TYPE_DOOR_R;
			}
			else if(vert->flag == VERT_FLAG_POLY_DOOR_VERT)
			{
				hole->body[vert->layer][hole->vert[vert->ref_no[4]].ref_no[4]].poly.color = POLY_TYPE_DOOR_U;
				hole->body[vert->layer][hole->vert[vert->ref_no[4]].ref_no[5]].poly.color = POLY_TYPE_DOOR_D;
			}
		}
	}
	else if(vert->ref_no[0] == 1)
	{  //move the doors to the center
		if(vert->ref_no[1] < vert->radius)
		{
			vert->ref_no[1] += vert->ref_no[3];

			if(vert->flag == VERT_FLAG_POLY_DOOR_HORIZ)
			{
				//enlarge left door poly toward center
				hole->body[vert->layer][vert->ref_no[6]].poly.p[2].x += vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[6]].poly.p[3].x += vert->ref_no[3];

				//enlarge right door poly toward center
				hole->body[vert->layer][vert->ref_no[7]].poly.p[0].x -= vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[7]].poly.p[1].x -= vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[7]].poly.p[4].x -= vert->ref_no[3];
			}
			else if(vert->flag == VERT_FLAG_POLY_DOOR_VERT)
			{
				//enlarge top door poly toward center
				hole->body[vert->layer][vert->ref_no[6]].poly.p[0].y += vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[6]].poly.p[3].y += vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[6]].poly.p[4].y += vert->ref_no[3];

				//enlarge bottom door poly toward center
				hole->body[vert->layer][vert->ref_no[7]].poly.p[1].y -= vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[7]].poly.p[2].y -= vert->ref_no[3];
			}
		}
		else
		{
			//set to normal walls now..
			hole->body[vert->layer][vert->ref_no[6]].poly.color = POLY_TYPE_STD_WALL;
			hole->body[vert->layer][vert->ref_no[7]].poly.color = POLY_TYPE_STD_WALL;
			vert->ref_no[0] = 2;
		}
	}
	else if(vert->ref_no[0] == 2)
	{  //stay closed for this duration
		if(++vert->ref_no[1] >= DEFAULT_DOOR_DURATION)
		{
			vert->ref_no[0] = 3;
			vert->ref_no[1] = 0;
		}
	}
	else if(vert->ref_no[0] == 3)
	{  //stay closed for this duration
		if(vert->ref_no[1] < vert->radius)
		{
			vert->ref_no[1] += vert->ref_no[3];

			if(vert->flag == VERT_FLAG_POLY_DOOR_HORIZ)
			{
				//enlarge left door poly toward center
				hole->body[vert->layer][vert->ref_no[6]].poly.p[2].x -= vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[6]].poly.p[3].x -= vert->ref_no[3];

				//enlarge right door poly toward center
				hole->body[vert->layer][vert->ref_no[7]].poly.p[0].x += vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[7]].poly.p[1].x += vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[7]].poly.p[4].x += vert->ref_no[3];
			}
			else if(vert->flag == VERT_FLAG_POLY_DOOR_VERT)
			{
				//enlarge top door poly toward center
				hole->body[vert->layer][vert->ref_no[6]].poly.p[0].y -= vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[6]].poly.p[3].y -= vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[6]].poly.p[4].y -= vert->ref_no[3];

				//enlarge bottom door poly toward center
				hole->body[vert->layer][vert->ref_no[7]].poly.p[1].y += vert->ref_no[3];
				hole->body[vert->layer][vert->ref_no[7]].poly.p[2].y += vert->ref_no[3];
			}
		}
		else
		{
			vert->ref_no[0] = 0;
			vert->ref_no[1] = 0;
		}
	}
}

//handle the various vert actions, depending on the vert's flag
void handle_vert_action(VERT * vert, HOLE * hole, COURSE_IMAGE_BANK * i_bank)
{
	int i;

	switch(vert->flag)
	{
		case VERT_FLAG_ANIMAL_0:
		{
			handle_animal_vert(vert, hole);
			break;
		}
		case VERT_FLAG_ANIMAL_1:
		{
			handle_animal_vert(vert, hole);
			break;
		}
		case VERT_FLAG_POLY_MORPH:
		{
			//rotate polygon..
			if(vert->ref_no[0] == 0)
			{
				POLY tmp_poly = hole->body[vert->layer][vert->ref_no[1]].poly;
				float rot_angle = (vert->ref_no[2] * ALLEGRO_PI) / 180.0;

				for(i = 0; i < tmp_poly.num_vertices; i++)
				{
					tmp_poly.p[i].x -= vert->s.x;
					tmp_poly.p[i].y -= vert->s.y;
					rotate_vertex(&tmp_poly.p[i], rot_angle);
				}
				for(i = 0; i < tmp_poly.num_vertices; i++)
				{
					tmp_poly.p[i].x += vert->s.x;
					tmp_poly.p[i].y += vert->s.y;
				}
				hole->body[vert->layer][vert->ref_no[1]].poly = tmp_poly;
			}
			calc_body_normals(&hole->body[vert->layer][vert->ref_no[1]]);
			break;
		}
		case VERT_FLAG_POLY_MOVE_HORIZ:  //this vert controls a poly that moves horizontally
		{
			if(!vert->ref_no[0])
			{  //move to right endpoint
				if(vert->ref_no[2] < vert->ref_no[4])
				{
					vert->ref_no[2] += vert->ref_no[5];
					translate_poly_x(&hole->body[vert->layer][vert->ref_no[1]].poly, vert->ref_no[5]);
				}
				else vert->ref_no[0] = 1;
			}
			else
			{                  //move to left endpoint
				if(vert->ref_no[2] > 0)
				{
					vert->ref_no[2] -= vert->ref_no[5];
					translate_poly_x(&hole->body[vert->layer][vert->ref_no[1]].poly, -vert->ref_no[5]);
				}
				else vert->ref_no[0] = 0;
			}
			break;
		}
		case VERT_FLAG_POLY_MOVE_VERT:  //this vert controls a poly that moves vertically
		{
			if(!vert->ref_no[0])
			{  //move to right endpoint
				if(vert->ref_no[2] < vert->ref_no[4])
				{
					vert->ref_no[2] += vert->ref_no[5];
					translate_poly_y(&hole->body[vert->layer][vert->ref_no[1]].poly, vert->ref_no[5]);
				}
				else vert->ref_no[0] = 1;
			}
			else
			{  //move back to vert center
				if(vert->ref_no[2] > 0)
				{
					vert->ref_no[2] -= vert->ref_no[5];
					translate_poly_y(&hole->body[vert->layer][vert->ref_no[1]].poly, -vert->ref_no[5]);
				}
				else vert->ref_no[0] = 0;
			}
			break;
		}
		case VERT_FLAG_TELEPORT:      //this vert acts as a teleport pad
		{
			if(i_bank->object_pic[3][0]->frames < 2) // animate only if first animation is static image
			{
				vert->ref_no[5] = !vert->ref_no[5];  //2-frame animation loop
			}
			break;
		}
		case VERT_FLAG_BUBBLE_HORIZ: //handle bubble motion
		case VERT_FLAG_BUBBLE_VERT:
		{
			handle_bubble_vert(vert, hole);
			break;
		}
		case VERT_FLAG_LASER_SEND:
		{  //this vert fires a laser to an end-vert
			if(!vert->ref_no[0])
			{  //laser inactive, run timer to delay ceiling
				if(vert->ref_no[1] < vert->ref_no[2]*2)
				{  //keep counting while the laser is being fired (multiply by two to slow it down a little)
					//vert->ref_no[1] += t;   //keep counting while laser is being fired (multiply by two to slow it down a bit)
					vert->ref_no[1]++;
					/* new stuff]] (changed the increment to just 1, t seems to make timing eratic..)*/
				}
				else
				{   //set laser to active
					vert->ref_no[1] = 0;     //reset the delay timer
					vert->ref_no[0] = hole->vert[vert->ref_no[5]].ref_no[0] = 1;   //reset the activtiy of the laser send and receive verts

					//poly that represents laser is set to detect collisions
					hole->body[vert->layer][vert->ref_no[4]].poly.color = POLY_TYPE_LASER_WALL;
				}
			}
			else
			{   //fire laser
				if(vert->ref_no[1] < vert->ref_no[3] * 2)
				{
					//vert->ref_no[1] += t;   //keep counting while laser is being fired (multiply by two to slow it down a bit)
					vert->ref_no[1]++;
				}
				else
				{   //deactivate laser
					vert->ref_no[1] = 0;    //reset the laser delay timer
					vert->ref_no[0] = hole->vert[vert->ref_no[5]].ref_no[0] = 0;   //reset the activity of the laser send and receive verts

					//poly that represents laser is set to ignore collisions
					hole->body[vert->layer][vert->ref_no[4]].poly.color = POLY_TYPE_EMPTY_WALL;
				}
			}
			break;
		}
		case VERT_FLAG_CONVEYOR:
		{   //animate the conveyor belt
			if(vert->ref_no[1])
			{
				if(vert->ref_no[4] < vert->ref_no[2] * 4 - 1)
				{
					//vert->ref_no[4] += t;
					vert->ref_no[4]++;
				}
				else
				{
					vert->ref_no[4] = 0;
				}
			}
			break;
		}
		case VERT_FLAG_POLY_DOOR_HORIZ:  //this vert controls a dual poly door
		case VERT_FLAG_POLY_DOOR_VERT:  //this vert controls a dual poly door
		{
			handle_door_vert(vert, hole);
			break;
		}
		case VERT_FLAG_CLOUD_HORIZ: //handle the cloud motion and anims
		case VERT_FLAG_CLOUD_VERT:  //animate the conveyor belt
		{
			handle_cloud_vert(vert, hole);
			break;
		}
		case VERT_FLAG_BLINKER:     //this vert is a blinking light
		{
			handle_blinker_vert(vert, hole);
			break;
		}
		default:
		{
			break;
		}
	}
	vert->tick++;
}

//vert logic..
void vert_logic(VERT * vert, HOLE * hole, COURSE_IMAGE_BANK * i_bank)
{
	handle_vert_action(vert, hole, i_bank);
}

//display one vert
void hole_vert_display(ALLEGRO_BITMAP * bp, VERT * vert, HOLE * hole, COURSE_IMAGE_BANK * i_bank, PALETTE * palette, float ox, float oy, float zoom)
{
	int i;
	float r, w, h;

	switch(vert->flag)
	{
		case VERT_FLAG_TEE:
			break;

		case VERT_FLAG_HOLE:        //draw the cup
			t3f_draw_scaled_animation(i_bank->vert_pic[vert->flag][vert->ref_no[0]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->vert_pic[vert->flag][vert->ref_no[0]]) / 2) * zoom - ox, (hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->vert_pic[vert->flag][vert->ref_no[0]]) / 2) * zoom - oy, 0, zoom, 0);
			break;

		case VERT_FLAG_TREE_0:      //draw the various tree types
			r = (float)vert->radius / (float)(vgolf_get_bitmap_width(i_bank->object_pic[0][0]) / 2);
			h = r * (float)vgolf_get_bitmap_height(i_bank->object_pic[0][0]);
			t3f_draw_scaled_animation(i_bank->object_pic[0][0], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vert->radius) * zoom - ox, (hole->s.y + vert->s.y - h / 2.0) * zoom - oy, 0, r * zoom, 0);
//			t3f_draw_animation(i_bank->object_pic[0][0], t3f_color_white, vert->tick, hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->object_pic[0][0]) / 2, hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->object_pic[0][0]) / 2, 0.0, 0);
			break;

		case VERT_FLAG_TREE_1:
			r = (float)vert->radius / (float)(vgolf_get_bitmap_width(i_bank->object_pic[1][0]) / 2);
			h = r * (float)vgolf_get_bitmap_height(i_bank->object_pic[1][0]);
			t3f_draw_scaled_animation(i_bank->object_pic[1][0], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vert->radius) * zoom - ox, (hole->s.y + vert->s.y - h / 2.0) * zoom - oy, 0, r * zoom, 0);
//			t3f_draw_animation(i_bank->object_pic[1][0], t3f_color_white, vert->tick, hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->object_pic[1][0]) / 2, hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->object_pic[1][0]) / 2, 0.0, 0);
			break;

		case VERT_FLAG_ANIMAL_0:    //draw the various animal types
			t3f_draw_scaled_animation(i_bank->animal_pic[0][vert->ref_no[0]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->animal_pic[0][vert->ref_no[0]]) / 2) * zoom - ox, (hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->animal_pic[0][vert->ref_no[0]]) / 2) * zoom - oy, 0, zoom, 0);
			break;

		case VERT_FLAG_ANIMAL_1:
			t3f_draw_scaled_animation(i_bank->animal_pic[1][vert->ref_no[0]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->animal_pic[1][vert->ref_no[0]]) / 2) * zoom - ox, (hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->animal_pic[1][vert->ref_no[0]]) / 2) * zoom - oy, 0, zoom, 0);
			break;

		case VERT_FLAG_POLY_MORPH:  //draw the rotating polygon to the poly layer
			draw_filled_polygon(hole->body[vert->layer][vert->ref_no[1]].poly, hole->s.x, hole->s.y, ox, oy, zoom, al_map_rgb(192, 192, 192));
			draw_polygon(hole->body[vert->layer][vert->ref_no[1]].poly, hole->s.x, hole->s.y, ox, oy, zoom, al_map_rgb(0, 0, 0));
//			floodfill(hole->layer_pic[2], hole->s.x+vert->s.x, hole->s.y+vert->s.y, vert->ref_no[3]);
			break;

		case VERT_FLAG_POLY_MOVE_HORIZ:  //this vert controls a poly that moves horizontally
			draw_filled_polygon(hole->body[vert->layer][vert->ref_no[1]].poly, hole->s.x, hole->s.y, ox, oy, zoom, al_map_rgb(192, 192, 192));
			draw_polygon(hole->body[vert->layer][vert->ref_no[1]].poly, hole->s.x, hole->s.y, ox, oy, zoom, al_map_rgb(0, 0, 0));
//			floodfill(hole->layer_pic[2], hole->s.x+vert->s.x+vert->ref_no[2], hole->s.y+vert->s.y, vert->ref_no[3]);
			break;

		case VERT_FLAG_POLY_MOVE_VERT:  //this vert controls a poly that moves horizontally
			draw_filled_polygon(hole->body[vert->layer][vert->ref_no[1]].poly, hole->s.x, hole->s.y, ox, oy, zoom, al_map_rgb(192, 192, 192));
			draw_polygon(hole->body[vert->layer][vert->ref_no[1]].poly, hole->s.x, hole->s.y, ox, oy, zoom, al_map_rgb(0, 0, 0));
//			floodfill(hole->layer_pic[2], hole->s.x+vert->s.x, hole->s.y+vert->s.y+vert->ref_no[2], vert->ref_no[3]);
			break;

		case VERT_FLAG_LASER_SEND:
			//draw the laser sender/receiver
			t3f_draw_scaled_animation(i_bank->object_pic[2][vert->ref_no[0]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->object_pic[2][0]) / 2) * zoom - ox, (hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->object_pic[2][0]) / 2) * zoom - oy, 0, zoom, 0);
			//if the laser is active, draw the laser beam between the vert endpoints
			break;

		case VERT_FLAG_LASER_RECV:
			//draw the laser sender/receiver
			t3f_draw_scaled_animation(i_bank->object_pic[2][vert->ref_no[0]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->object_pic[2][0]) / 2) * zoom - ox, (hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->object_pic[2][0]) / 2) * zoom - oy, 0, zoom, 0);
			break;

		case VERT_FLAG_TELEPORT:   //draw the teleport pad
			t3f_draw_scaled_animation(i_bank->object_pic[3][vert->ref_no[5]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - vgolf_get_bitmap_width(i_bank->object_pic[3][0]) / 2) * zoom - ox, (hole->s.y + vert->s.y - vgolf_get_bitmap_height(i_bank->object_pic[3][0]) / 2) * zoom - oy, 0, zoom, 0);
			break;

		case VERT_FLAG_CONVEYOR:   //draw the conveyor belt
			w = (int)vgolf_get_bitmap_width(i_bank->object_pic[4][0]) / 2;
			h = (int)vgolf_get_bitmap_height(i_bank->object_pic[4][0]) / 2;

			/* vertical */
			if(vert->ref_no[0] < 2)
			{
				for(i = 0; i < (vert->radius * 2) / (int)vgolf_get_bitmap_height(i_bank->object_pic[4][0]); i++)
				{
					if(vert->ref_no[0] == 0)
					{
						t3f_draw_scaled_animation(i_bank->object_pic[4][vert->ref_no[4] / vert->ref_no[2]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - w) * zoom - ox, (hole->s.y + vert->s.y - vert->radius + i * (int)vgolf_get_bitmap_height(i_bank->object_pic[4][0])) * zoom - oy, 0, zoom, 0);
					}
					else
					{
						t3f_draw_scaled_animation(i_bank->object_pic[4][vert->ref_no[4] / vert->ref_no[2]], t3f_color_white, vert->tick, (hole->s.x + vert->s.x - w) * zoom - ox, (hole->s.y + vert->s.y - vert->radius + i * (int)vgolf_get_bitmap_height(i_bank->object_pic[4][0])) * zoom - oy, 0, zoom, ALLEGRO_FLIP_VERTICAL);
					}
				}
			}

			/* horizontal */
			else
			{
				for(i = 0; i < (vert->radius * 2) / (int)vgolf_get_bitmap_height(i_bank->object_pic[4][0]); i++)
				{
					if(vert->ref_no[0] == 3)
					{
						t3f_draw_rotated_scaled_animation(i_bank->object_pic[4][vert->ref_no[4] / vert->ref_no[2]], t3f_color_white, vert->tick, w, h, (hole->s.x + vert->s.x + w - vert->radius + i * (int)vgolf_get_bitmap_height(i_bank->object_pic[4][0])) * zoom - ox, (hole->s.y + vert->s.y) * zoom - oy, 0, ALLEGRO_PI / 2.0, zoom, 0);
					}
					else
					{
						t3f_draw_rotated_scaled_animation(i_bank->object_pic[4][vert->ref_no[4] / vert->ref_no[2]], t3f_color_white, vert->tick, w, h, (hole->s.x + vert->s.x + w - vert->radius + i * (int)vgolf_get_bitmap_height(i_bank->object_pic[4][0])) * zoom - ox, (hole->s.y + vert->s.y) * zoom - oy, 0, ALLEGRO_PI / 2.0, zoom, ALLEGRO_FLIP_VERTICAL);
					}
				}
			}
			break;

		case VERT_FLAG_CLOUD_HORIZ:  //draw the cloud to the poly layer (so it flies overhead..)
		case VERT_FLAG_CLOUD_VERT:  //draw the cloud to the poly layer (so it flies overhead..)
			al_draw_filled_circle((hole->s.x + vert->s.x) * zoom - ox, (hole->s.y + vert->s.y) * zoom - oy, vert->radius * zoom, get_color(palette, vert->ref_no[5]));
			al_draw_circle((hole->s.x + vert->s.x) * zoom - ox, (hole->s.y + vert->s.y) * zoom - oy, vert->radius * zoom, get_color(palette, vert->ref_no[5]-1), VERT_POLY_LINE_THICKNESS);
			break;

		case VERT_FLAG_BLINKER:     //draw the flashing blinker light..
			if(vert->ref_no[0])
			{
				if(vert->ref_no[4]) al_draw_filled_rectangle((hole->s.x + vert->s.x - vert->radius) * zoom - ox, (hole->s.y + vert->s.y - vert->radius) * zoom - oy, (hole->s.x + vert->s.x + vert->radius) * zoom - ox, (hole->s.y + vert->s.y + vert->radius) * zoom - oy, get_color(palette, vert->ref_no[5]));
				else al_draw_filled_circle((hole->s.x + vert->s.x) * zoom - ox, (hole->s.y + vert->s.y) * zoom - oy, vert->radius * zoom, get_color(palette, vert->ref_no[5]));
			}
			break;

		case VERT_FLAG_POLY_DOOR_HORIZ:  //draw the moving doors
		case VERT_FLAG_POLY_DOOR_VERT:  //draw the moving doors
		{
			int lp_no = vert->ref_no[6];
			int rp_no = vert->ref_no[7];
			al_draw_filled_rectangle((hole->s.x + hole->body[vert->layer][lp_no].poly.p[1].x) * zoom - ox, (hole->s.y + hole->body[vert->layer][lp_no].poly.p[1].y) * zoom - oy, (hole->s.x + hole->body[vert->layer][lp_no].poly.p[3].x) * zoom - ox, (hole->s.y + hole->body[vert->layer][lp_no].poly.p[3].y) * zoom - oy, get_color(palette, vert->ref_no[5]));
			al_draw_rectangle((hole->s.x + hole->body[vert->layer][lp_no].poly.p[1].x) * zoom - ox + 0.5, (hole->s.y + hole->body[vert->layer][lp_no].poly.p[1].y) * zoom - oy + 0.5, (hole->s.x + hole->body[vert->layer][lp_no].poly.p[3].x) * zoom - ox + 0.5, (hole->s.y + hole->body[vert->layer][lp_no].poly.p[3].y) * zoom - oy + 0.5, get_color(palette, 1), 1.0);
			al_draw_filled_rectangle((hole->s.x + hole->body[vert->layer][rp_no].poly.p[1].x) * zoom - ox, (hole->s.y + hole->body[vert->layer][rp_no].poly.p[1].y) * zoom - oy, (hole->s.x + hole->body[vert->layer][rp_no].poly.p[3].x) * zoom - ox, (hole->s.y + hole->body[vert->layer][rp_no].poly.p[3].y) * zoom - oy, get_color(palette, vert->ref_no[5]));
			al_draw_rectangle((hole->s.x + hole->body[vert->layer][rp_no].poly.p[1].x) * zoom - ox + 0.5, (hole->s.y + hole->body[vert->layer][rp_no].poly.p[1].y) * zoom - oy + 0.5, (hole->s.x + hole->body[vert->layer][rp_no].poly.p[3].x) * zoom - ox + 0.5, (hole->s.y + hole->body[vert->layer][rp_no].poly.p[3].y) * zoom - oy + 0.5, get_color(palette, 1), VERT_POLY_LINE_THICKNESS);
			break;
		}

		case VERT_FLAG_BUBBLE_HORIZ:
		case VERT_FLAG_BUBBLE_VERT:
		{
			ALLEGRO_COLOR color;

			if(al_get_new_bitmap_flags() & ALLEGRO_NO_PREMULTIPLIED_ALPHA)
			{
				color = al_map_rgba_f(1.0, 1.0, 1.0, 0.5);
			}
			else
			{
				color = al_map_rgba_f(0.5, 0.5, 0.5, 0.5);
			}
			//drawing_mode(DRAW_MODE_TRANS, 0, 0, 0);
			al_draw_circle((hole->s.x + vert->s.x) * zoom - ox, (hole->s.y + vert->s.y) * zoom - oy, vert->radius * zoom, color, VERT_POLY_LINE_THICKNESS);
			al_draw_filled_circle((hole->s.x + vert->s.x - vert->radius / 2) * zoom - ox, (hole->s.y + vert->s.y - vert->radius / 2) * zoom - oy, (vert->radius / 5.0) * zoom, color);
			//solid_mode();
			break;
		}
		default:
			break;
	}
}

//go through the display code for all verts in this hole
void hole_vert_display_bitmap(ALLEGRO_BITMAP * bp, HOLE * hole, int layer, COURSE_IMAGE_BANK * i_bank, PALETTE * palette, float ox, float oy, float zoom)
{
	int i;

	for(i = 0; i < hole->num_verts; i++)
	{
		switch(hole->vert[i].flag)
		{
			case VERT_FLAG_TEE:
			case VERT_FLAG_HOLE:
			case VERT_FLAG_TREE_0:      //draw the various tree types
			case VERT_FLAG_TREE_1:
			case VERT_FLAG_ANIMAL_0:    //draw the various animal types
			case VERT_FLAG_ANIMAL_1:
			case VERT_FLAG_LASER_SEND:
			case VERT_FLAG_LASER_RECV:
			case VERT_FLAG_TELEPORT:   //draw the teleport pad
			case VERT_FLAG_CONVEYOR:   //draw the conveyor belt
			{
				if(hole->vert[i].layer == layer)
				{
					hole_vert_display(bp, &hole->vert[i], hole, i_bank, palette, ox, oy, zoom);
				}
				break;
			}
		}
	}
}

void hole_vert_display_single_primitive(ALLEGRO_BITMAP * bp, HOLE * hole, int i, COURSE_IMAGE_BANK * i_bank, PALETTE * palette, float ox, float oy, float zoom)
{
	switch(hole->vert[i].flag)
	{
		case VERT_FLAG_TEE:
		case VERT_FLAG_HOLE:
		case VERT_FLAG_TREE_0:      //draw the various tree types
		case VERT_FLAG_TREE_1:
		case VERT_FLAG_ANIMAL_0:    //draw the various animal types
		case VERT_FLAG_ANIMAL_1:
		case VERT_FLAG_LASER_RECV:
		case VERT_FLAG_TELEPORT:   //draw the teleport pad
		case VERT_FLAG_CONVEYOR:   //draw the conveyor belt
		{
			break;
		}
		case VERT_FLAG_LASER_SEND:
		{
			if(hole->vert[i].ref_no[0])
			{
				//laser_display(hole->layer_pic[2], hole->s.x+vert->s.x, hole->s.y+vert->s.y, hole->s.x+hole->vert[vert->ref_no[5]].s.x, hole->s.y+hole->vert[vert->ref_no[5]].s.y, hole->vert[vert->ref_no[5]].ref_no[1]);
				al_draw_line((hole->s.x + hole->vert[i].s.x) * zoom - ox + 0.5, (hole->s.y + hole->vert[i].s.y) * zoom - oy + 0.5, (hole->s.x + hole->vert[hole->vert[i].ref_no[5]].s.x) * zoom - ox + 0.5, (hole->s.y + hole->vert[hole->vert[i].ref_no[5]].s.y) * zoom - oy + 0.5, get_color(palette, hole->vert[hole->vert[i].ref_no[5]].ref_no[1]), VERT_POLY_LINE_THICKNESS);
			}
			break;
		}
		default:
		{
			hole_vert_display(bp, &hole->vert[i], hole, i_bank, palette, ox, oy, zoom);
			break;
		}
	}
}

void hole_vert_display_primitive(ALLEGRO_BITMAP * bp, HOLE * hole, int layer, COURSE_IMAGE_BANK * i_bank, PALETTE * palette, float ox, float oy, float zoom)
{
	int i;

	for(i = 0; i < hole->num_verts; i++)
	{
		if(hole->vert[i].layer == layer)
		{
			hole_vert_display_single_primitive(bp, hole, i, i_bank, palette, ox, oy, zoom);
		}
	}
}
