#include "t3f/t3f.h"

#include "main.h"

#include "ui.h"
#include "ui_dialog_proc.h"
#include "ui_poly.h"
#include "ui_vert.h"
#include "ui_image.h"
#include "draw.h"
#include "init.h"
#include "ui_menu_proc.h"
#include "ui_shortcuts.h"

void crsed_event_proc(ALLEGRO_EVENT * event, void * data)
{
	switch(event->type)
	{
		case ALLEGRO_EVENT_DISPLAY_RESIZE:
		{
			crsed_center_dialogs(data, event->display.width, event->display.height);
			break;
		}
		case ALLEGRO_EVENT_DISPLAY_CLOSE:
		{
			if(t3f_menus_enabled())
			{
				crsed_menu_file_exit(0, data);
			}
			return;
		}
		case T3GUI_EVENT_CLOSE:
		{
			crsed_dialog_event_proc(event, data);
			break;
		}
		case T3GUI_EVENT_CANCEL:
		{
			/* if we are closing the last dialog, re-enable menus */
			if(t3gui_get_active_dialogs() <= 1)
			{
				t3f_enable_menus(true);
			}
			break;
		}
	}
	t3f_event_handler(event);
}

static void crsed_mouse_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	float mx, my;

	app->old_mouse_x = app->mouse_x;
	app->old_mouse_y = app->mouse_y;
	app->old_mouse_b = app->mouse_b;
	mx = ((float)(t3f_real_mouse_x - app->display_offset_x) / app->display_scale_x);
	my = ((float)(t3f_real_mouse_y - app->display_offset_y) / app->display_scale_y);
	app->mouse_x = mx / app->zoom + (app->ox / app->zoom);
	app->mouse_y = my / app->zoom + (app->oy / app->zoom);
	app->mouse_b = (t3f_mouse_button[0] ? 1 : 0) | (t3f_mouse_button[1] ? 2 : 0);

}

void crsed_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	if(t3gui_get_active_dialogs())
	{
		t3gui_logic();
		return; // skip the rest of the logic if a dialog is active
	}

	crsed_process_shortcuts(data);
	crsed_mouse_logic(data);
	if(app->course)
	{
		switch(app->mode)
		{
			case CRSED_STATE_POLYGONS:
			{
				crsed_poly_logic(data);
				break;
			}
			case CRSED_STATE_OBJECTS:
			{
				crsed_vert_logic(data);
				break;
			}
			case CRSED_STATE_IMAGES:
			{
				crsed_image_logic(data);
				break;
			}
		}
	}
}

void crsed_render(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	crsed_setup_transformation(data);
	al_use_transform(&app->transform);
	crsed_set_clipping_rectangle(&app->transform, 0, 0, 640, 480);
	al_clear_to_color(al_map_rgb(0, 0, 64));
	if(app->course)
	{
		switch(app->mode)
		{
			case CRSED_STATE_POLYGONS:
			{
				crsed_poly_render(data);
				break;
			}
			case CRSED_STATE_OBJECTS:
			{
				crsed_vert_render(data);
				break;
			}
			case CRSED_STATE_IMAGES:
			{
				crsed_image_render(data);
				break;
			}
		}
	}
	crsed_render_status(data);
	t3gui_render();
}

int main(int argc, char * argv[])
{
	APP_INSTANCE app;
	char buffer[1024];

	if(app_initialize(&app, argc, argv))
	{
		t3f_run();
	}
	else
	{
		printf("Error: could not initialize T3F!\n");
	}
	t3gui_exit();
	return 0;
}
