#include "t3f/t3f.h"
#include "t3f/menu.h"
#include "ui.h"
#include "ui_dialog_proc.h"
#include "instance.h"
#include "main.h"
#include "tables.h"
#include "ui_vert.h"
#include "ui_menu_proc.h"

char crsed_edit_text[4][1024] = {{0}};

void crsed_fix_window_title(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	char window_text[1024] = {0};
	ALLEGRO_PATH * pp;

	if(strcmp(app->loaded_filename, ""))
	{
		pp = al_create_path(app->loaded_filename);
		if(pp)
		{
			sprintf(window_text, "%svGolf Course Editor - %s (%s)", app->changes ? "*" : "", al_get_path_filename(pp), app->course->name);
			al_destroy_path(pp);
		}
	}
	else
	{
		sprintf(window_text, "%svGolf Course Editor - Untitled (%s)", app->changes ? "*" : "", app->course->name);
	}
	al_set_window_title(t3f_display, window_text);
}

void crsed_mark_change(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	app->changes++;
	crsed_fix_window_title(data);
}

void crsed_set_cursor(void * data, ALLEGRO_SYSTEM_MOUSE_CURSOR cursor)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	if(app->current_cursor != cursor)
	{
		al_set_system_mouse_cursor(t3f_display, cursor);
		app->current_cursor = cursor;
	}
}

void crsed_center_dialogs(APP_INSTANCE * app, int w, int h)
{
    t3gui_center_dialog(app->palette_dialog, w, h);
    t3gui_center_dialog(app->timer_dialog, w, h);
    t3gui_center_dialog(app->timer2_dialog, w, h);
    t3gui_center_dialog(app->speed_dialog, w, h);
    t3gui_center_dialog(app->speed2_dialog, w, h);
    t3gui_center_dialog(app->tags_dialog, w, h);
    t3gui_center_dialog(app->direction_dialog, w, h);
    t3gui_center_dialog(app->size_dialog, w, h);
    t3gui_center_dialog(app->layer_dialog, w, h);
    t3gui_center_dialog(app->target_layer_dialog, w, h);
	t3gui_center_dialog(app->type_dialog, w, h);
    t3gui_center_dialog(app->par_dialog, w, h);
    t3gui_center_dialog(app->unsaved_dialog, w, h);
    t3gui_center_dialog(app->about_dialog, w, h);
}

void crsed_render_status(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	ALLEGRO_TRANSFORM identity;
	ALLEGRO_STATE old_state;
	int type_index = 0;
	int i;
	float y;

	al_store_state(&old_state, ALLEGRO_STATE_TRANSFORM);
	al_identity_transform(&identity);
	al_use_transform(&identity);
	al_set_clipping_rectangle(0, 0, al_get_display_width(t3f_display), al_get_display_height(t3f_display));
	y = al_get_display_height(t3f_display) - 16;
	al_draw_filled_rectangle(0, y, al_get_display_width(t3f_display), al_get_display_height(t3f_display), al_map_rgb(227, 227, 227));
	if(app->course)
	{
		if(app->mode == CRSED_STATE_OBJECTS && app->vert_mode == 1)
		{
			al_draw_textf(app->font, al_map_rgb(0, 0, 0), 2, y, 0, "Click object to connect to...");
		}
		else if(app->mode == CRSED_STATE_OBJECTS && app->vert_mode == 2)
		{
			al_draw_textf(app->font, al_map_rgb(0, 0, 0), 2, y, 0, "Click to set angle...");
		}
		else if(app->mode == CRSED_STATE_IMAGES)
		{
			if(!(*app->current_image))
			{
				al_draw_textf(app->font, al_map_rgb(0, 0, 0), 2, y, 0, "Image: %s", crsed_image_type_name[app->selected_image]);
			}
			else
			{
				al_draw_textf(app->font, al_map_rgb(0, 0, 0), 2, y, 0, "Image: %s (%dx%d)",  crsed_image_type_name[app->selected_image], (int)(*app->current_image)->frame[0]->width,  (int)(*app->current_image)->frame[0]->height);
			}
		}
		else if(app->mode == CRSED_STATE_BUILD)
		{

		}
		else
		{
			if(app->course->num_holes <= 0)
			{
				al_draw_textf(app->font, al_map_rgb(0, 0, 0), 2, y, 0, "No holes. Use \"Hole->Add\" to create a hole.");
			}
			else
			{
				al_draw_textf(app->font, al_map_rgb(0, 0, 0), 2, y, 0, "Hole: %d of %d (Par: %d)", app->selected_hole + 1, app->course->num_holes, app->course->hole[app->selected_hole].par_score);
				if(app->mode == CRSED_STATE_POLYGONS)
				{
					al_draw_textf(app->font, al_map_rgb(0, 0, 0), al_get_display_width(t3f_display) - 2, y, ALLEGRO_ALIGN_RIGHT, "Layer: %d - Polygon: %d (Vertex: %d)", app->selected_layer, app->selected_poly, app->selected_point);
				}
				else
				{
					for(i = 0; i < 39; i++)
					{
						if(crsed_object_type[i] == app->course->hole[app->selected_hole].vert[app->selected_vert].flag)
						{
							type_index = i;
							break;
						}
					}
					al_draw_textf(app->font, al_map_rgb(0, 0, 0), al_get_display_width(t3f_display) - 2, y, ALLEGRO_ALIGN_RIGHT, "Object: %d (Radius: %.2f) (Type: %s)", app->selected_vert, app->course->hole[app->selected_hole].vert[app->selected_vert].radius, crsed_object_type_name[type_index]);
				}
			}
		}
	}
	al_restore_state(&old_state);
}

void draw_handle(void * data, ALLEGRO_BITMAP * bp, int x, int y, ALLEGRO_COLOR color)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;
	int o = app->zoom / 2;
	int peg_size = al_get_bitmap_width(app->peg_bitmap);
	int peg_offset = peg_size / 2;
	ALLEGRO_STATE old_state;
	ALLEGRO_TRANSFORM identity;

	al_store_state(&old_state, ALLEGRO_STATE_TRANSFORM);
	al_identity_transform(&identity);
	al_use_transform(&identity);
	t3f_draw_bitmap(app->peg_bitmap, color, (x * app->zoom - app->ox + o) * app->display_scale_x + app->display_offset_x - peg_offset, (y * app->zoom - app->oy + o) * app->display_scale_y + app->display_offset_y - peg_offset, 0, 0);
	al_restore_state(&old_state);
}

void crsed_camera_logic(void * data)
{
	APP_INSTANCE * app = (APP_INSTANCE *)data;

	if(app->zoom > 1)
	{
		if(t3f_mouse_x < 16)
		{
			app->ox -= app->zoom;
			if(app->ox < 0)
			{
				app->ox = 0;
			}
		}
		else if(t3f_mouse_x >= 640 - 16)
		{
			app->ox += app->zoom;
			if(app->ox > 640 * app->zoom - 640)
			{
				app->ox = 640 * app->zoom - 640;
			}
		}
		if(t3f_mouse_y > 16 && t3f_mouse_y < 16 + 16)
		{
			app->oy -= app->zoom;
			if(app->oy < 0)
			{
				app->oy = 0;
			}
		}
		else if(t3f_mouse_y >= 480 + 16 - 16)
		{
			app->oy += app->zoom;
			if(app->oy > 480 * app->zoom - 480)
			{
				app->oy = 480 * app->zoom - 480;
			}
		}
	}
}
